﻿/*
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using Canonical.UbuntuOne.Common.Win32;
using Microsoft.Win32;
using NUnit.Framework;
using Rhino.Mocks;

namespace Canonical.Ubuntu.SSO.Tests
{
    [TestFixture]
    public class KeyringFixture
    {
        #region Variables

        private MockRepository _mocks;
        private IRegistryKey _userRegistry;
        private IRegistryKey _keyringsRoot;
        private IRegistryKey _keyringKey;
        private IDataProtector _dataProtector;
        private Keyring _keyring;

        #endregion

        #region Setup & TearDown

        [SetUp]
        public void Setup()
        {
            _mocks = new MockRepository();
            _userRegistry = _mocks.StrictMock<IRegistryKey>();
            _keyringsRoot = _mocks.StrictMock<IRegistryKey>();
            _keyringKey = _mocks.StrictMock<IRegistryKey>();
            _dataProtector = _mocks.StrictMock<IDataProtector>();
            _keyring = new Keyring(_userRegistry) { DataProtector = _dataProtector };
        }

        #endregion

        #region Tests

        [Test]
        public void DisposeTest()
        {
            using (_mocks.Record())
            {
                Expect.Call(() => _userRegistry.Dispose())
                    .Repeat.Once();
            }
            using (_mocks.Playback())
            {
                // we ensure that we always dispose the registry key to ensure no 
                // memory leaks.
                _keyring.Dispose();
            }
        }

        [TestCase("Default", "UbuntuOne", "myPassword")]
        [TestCase("Default", "Mandel", "Secret")]
        [TestCase("Test", "Leo", "MySecret")]
        public void CreateSecretNoRootPathPresentTest(string keyringName, string applicationName, string secret)
        {
            using (_mocks.Record())
            {
                Expect.Call(_userRegistry.GetSubKeyNames())
                    .Return(new string[0])
                    .Repeat.Any();

                Expect.Call(_userRegistry.CreateSubKey(Keyring.RootPath))
                    .Return(_keyringsRoot)
                    .Repeat.Once();

                Expect.Call(_keyringsRoot.CreateSubKey(keyringName))
                    .Return(_keyringKey)
                    .Repeat.Once();

                Expect.Call(_userRegistry.OpenSubKey(Keyring.RootPath, RegistryKeyPermissionCheck.ReadWriteSubTree))
                    .Return(_keyringsRoot)
                    .Repeat.Once();

                Expect.Call(_keyringsRoot.OpenSubKey(keyringName, RegistryKeyPermissionCheck.ReadWriteSubTree))
                    .Return(_keyringKey)
                    .Repeat.Once();

                Expect.Call(_dataProtector.Protect("", new byte[0], DataProtectionScope.CurrentUser))
                    .Return(new byte[0])
                    .IgnoreArguments()
                    .Repeat.Once();

                Expect.Call(() => _keyringKey.SetValue(applicationName, new byte[0]))
                    .Repeat.Once();

                Expect.Call(() => _keyringKey.Dispose())
                    .Repeat.Any();

                Expect.Call(() => _keyringsRoot.Dispose())
                    .Repeat.Any();

                Expect.Call(() => _userRegistry.Dispose())
                    .Repeat.Any();
            }
            using (_mocks.Playback())
            {
                _keyring.CreateSecret(keyringName, applicationName, secret);
            }
        }

        [TestCase("Default", "UbuntuOne", "myPassword")]
        [TestCase("Default", "Mandel", "Secret")]
        [TestCase("Test", "Leo", "MySecret")]
        public void CreateSecretNoKeyringPresentTest(string keyringName, string applicationName, string secret)
        {
            using (_mocks.Record())
            {
                Expect.Call(_userRegistry.GetSubKeyNames())
                    .Return(new[] { "Blah", "BlahBlah", Keyring.RootPath })
                    .Repeat.Any();
                Expect.Call(_userRegistry.CreateSubKey(Keyring.RootPath))
                    .Return(_keyringsRoot)
                    .Repeat.Once();
                Expect.Call(_keyringsRoot.CreateSubKey(keyringName))
                    .Return(_keyringKey)
                    .Repeat.Once();
                Expect.Call(_userRegistry.OpenSubKey(Keyring.RootPath, RegistryKeyPermissionCheck.ReadWriteSubTree))
                    .Return(_keyringsRoot)
                    .Repeat.Once();
                Expect.Call(_dataProtector.Protect(secret, Encoding.UTF8.GetBytes(Keyring.Entropy), DataProtectionScope.CurrentUser))
                    .Return(new byte[0])
                    .Repeat.Once();
                Expect.Call(_keyringsRoot.OpenSubKey(keyringName, RegistryKeyPermissionCheck.ReadWriteSubTree))
                    .Return(_keyringKey)
                    .Repeat.Once();
                Expect.Call(() => _keyringKey.SetValue(applicationName, new byte[0]))
                    .Repeat.Once();
                Expect.Call(() => _keyringKey.Dispose())
                    .Repeat.Any();
                Expect.Call(() => _keyringsRoot.Dispose())
                    .Repeat.Any();
                Expect.Call(() => _userRegistry.Dispose())
                    .Repeat.Any();
            }
            using (_mocks.Playback())
            {
                _keyring.CreateSecret(keyringName, applicationName, secret);
            }
        }

        [TestCase("Default", "UbuntuOne", "myPassword")]
        [TestCase("Default", "Mandel", "Secret")]
        [TestCase("Test", "Leo", "MySecret")]
        public void CreateSecretPresentKeyringTest(string keyringName, string applicationName, string secret)
        {
            using (_mocks.Record())
            {
                Expect.Call(_userRegistry.GetSubKeyNames())
                    .Return(new[] {"Blah", "BlahBlah", "Canonical", "Keyrings"})
                    .Repeat.Any();
                Expect.Call(_userRegistry.OpenSubKey("Canonical"))
                    .Return(_userRegistry)
                    .Repeat.Once();
                Expect.Call(_userRegistry.OpenSubKey(Keyring.RootPath))
                    .Return(_keyringsRoot)
                    .Repeat.Once();
                Expect.Call(_keyringsRoot.GetSubKeyNames())
                    .Return(new[] {keyringName})
                    .Repeat.Once();
                Expect.Call(_userRegistry.OpenSubKey(Keyring.RootPath, RegistryKeyPermissionCheck.ReadWriteSubTree))
                    .Return(_keyringsRoot)
                    .Repeat.Once();
                Expect.Call(_keyringsRoot.OpenSubKey(keyringName, RegistryKeyPermissionCheck.ReadWriteSubTree))
                    .Return(_keyringKey)
                    .Repeat.Once();
                Expect.Call(_dataProtector.Protect(secret, Encoding.UTF8.GetBytes(Keyring.Entropy), DataProtectionScope.CurrentUser))
                    .Return(new byte[0])
                    .Repeat.Once();
                Expect.Call(() => _keyringKey.SetValue(applicationName, new byte[0]))
                    .Repeat.Once();
                Expect.Call(() => _keyringKey.Dispose())
                    .Repeat.Any();
                Expect.Call(() => _keyringsRoot.Dispose())
                    .Repeat.Any();
                Expect.Call(() => _userRegistry.Dispose())
                    .Repeat.Any();
            }
            using (_mocks.Playback())
            {
                _keyring.CreateSecret(keyringName, applicationName, secret);
            }
        }

        [TestCase("Default", "UbuntuOne")]
        [TestCase("Keyring", "AppNAme")]
        public void GetSecretByNameNotKeyringTest(string keyringName, string applicationName)
        {
            using (_mocks.Record())
            {
                Expect.Call(_userRegistry.GetSubKeyNames())
                        .Return(new[] { "Blah", "BlahBlah" })
                        .Repeat.Once();
            }
            using (_mocks.Playback())
            {
                var secret = _keyring.GetSecretByName(keyringName, applicationName);
                Assert.IsNull(secret);
            }
        }

        [TestCase("Default", "UbuntuOne")]
        [TestCase("Keyring", "AppNAme")]
        public void GetSecretByNameKeyringExistsTest(string keyringName, string applicationName)
        {
            using (_mocks.Record())
            {
                Expect.Call(_userRegistry.GetSubKeyNames())
                    .Return(new[] {"Blah", "BlahBlah", "Canonical", "Keyrings"})
                    .Repeat.Any();
                Expect.Call(_userRegistry.OpenSubKey("Canonical"))
                    .Return(_userRegistry)
                    .Repeat.Once();
                Expect.Call(_userRegistry.OpenSubKey(Keyring.RootPath))
                    .Return(_keyringsRoot)
                    .Repeat.Once();
                Expect.Call(_keyringsRoot.GetSubKeyNames())
                    .Return(new[] {keyringName})
                    .Repeat.Once();
                Expect.Call(_userRegistry.OpenSubKey(Keyring.RootPath, RegistryKeyPermissionCheck.ReadWriteSubTree))
                    .Return(_keyringsRoot)
                    .Repeat.Once();
                Expect.Call(_keyringsRoot.OpenSubKey(keyringName, RegistryKeyPermissionCheck.ReadWriteSubTree))
                    .Return(_keyringKey)
                    .Repeat.Once();
                Expect.Call(_keyringKey.GetValue(applicationName))
                    .Return(new byte[0])
                    .Repeat.Once();
                Expect.Call(_dataProtector.Unprotect(new byte[0], new byte[0], DataProtectionScope.CurrentUser))
                    .IgnoreArguments()
                    .Return(applicationName)
                    .Repeat.Once();
                Expect.Call(() => _userRegistry.Dispose())
                    .Repeat.Any();
                Expect.Call(() => _keyringsRoot.Dispose())
                    .Repeat.Any();
                Expect.Call(() => _keyringKey.Dispose())
                    .Repeat.Any();
            }
            using (_mocks.Playback())
            {
                var secret = _keyring.GetSecretByName(keyringName, applicationName);
                Assert.AreEqual(applicationName, secret);
            }
        }

        [Test]
        public void GetKeyringsTest()
        {
            var keyrings = new[] { "Default", "Test", "Test1" };

            using (_mocks.Record())
            {
                // make it such that the root is present
                Expect.Call(_userRegistry.GetSubKeyNames())
                    .Return(new[] {"Blah", "BlahBlah", "Canonical", "Keyrings"})
                    .Repeat.Any();
                Expect.Call(_userRegistry.OpenSubKey("Canonical"))
                    .Return(_userRegistry)
                    .Repeat.Once();
                Expect.Call(_userRegistry.OpenSubKey(Keyring.RootPath))
                    .Return(_keyringsRoot)
                    .Repeat.Once();
                Expect.Call(_keyringsRoot.GetSubKeyNames())
                    .Return(keyrings)
                    .Repeat.Once();
                Expect.Call(() => _userRegistry.Dispose())
                    .Repeat.Any();
                Expect.Call(() => _keyringsRoot.Dispose())
                    .Repeat.Any();

            }
            using (_mocks.Playback())
            {
                Assert.AreSame(_keyring.GetKeyrings(), keyrings);
            }
        }

        [Test]
        public void GetKeyringsNotRootTest()
        {
            using (_mocks.Record())
            {
                // make it such that the root is present
                Expect.Call(_userRegistry.GetSubKeyNames())
                        .Return(new[] { "Blah", "BlahBlah" })
                        .Repeat.Once();
            }
            using (_mocks.Playback())
            {
                Assert.AreEqual(0, _keyring.GetKeyrings().Count());
            }
        }

        [TestCase("Default")]
        [TestCase("Test")]
        public void GetApplicationsTest(string keyringName)
        {
            var applications = new[] { "Ubuntu One", "Test", "Default" };
            using (_mocks.Record())
            {
                Expect.Call(_userRegistry.GetSubKeyNames())
                    .Return(new[] { "Blah", "BlahBlah", "Canonical", "Keyrings" })
                        .Repeat.Any();

                Expect.Call(_userRegistry.OpenSubKey("Canonical"))
                    .Return(_userRegistry)
                    .Repeat.Once();

                Expect.Call(_userRegistry.OpenSubKey("Keyrings"))
                    .Return(_keyringsRoot)
                    .Repeat.Any();

                Expect.Call(_userRegistry.OpenSubKey(Keyring.RootPath))
                    .Return(_keyringsRoot)
                    .Repeat.Twice();

                Expect.Call(_keyringsRoot.GetSubKeyNames())
                    .Return(new[] { keyringName })
                    .Repeat.Once();

                Expect.Call(_keyringsRoot.OpenSubKey(keyringName))
                    .Return(_keyringKey)
                    .Repeat.Once();

                Expect.Call(_keyringKey.GetSubKeyNames())
                    .Return(applications)
                    .Repeat.Once();

                Expect.Call(() => _userRegistry.Dispose())
                    .Repeat.Any();

                Expect.Call(() => _keyringKey.Dispose())
                    .Repeat.Any();

                Expect.Call(() => _keyringsRoot.Dispose())
                    .Repeat.Any();
            }
            using (_mocks.Playback())
            {
                Assert.AreSame(applications, _keyring.GetApplications(keyringName));
            }
        }

        [Test]
        public void GetApplicationNotKeyringTest()
        {
            using (_mocks.Record())
            {
                // make it such that the root is present
                Expect.Call(_userRegistry.GetSubKeyNames())
                        .Return(new[] { "Blah", "BlahBlah" })
                        .Repeat.Once();
            }
            using (_mocks.Playback())
            {
                Assert.AreEqual(0, _keyring.GetApplications("Test").Count());
            }
        }

        #endregion
    }
}
