﻿/*
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System.Security.Cryptography;
using System.Text;
using log4net;

namespace Canonical.Ubuntu.SSO
{
    /// <summary>
    /// Implementation of the IDataProtector that uses the DAPI api to perform the protection.
    /// </summary>
    public class DPAPIDataProtector : IDataProtector
    {
        #region Variables

        private ILog _logger;
        private readonly object _loggerLock = new object();

        #endregion

        #region DI properties

        /// <summary>
        /// Gets and sets the logger that will be used to log the operation of the
        /// class.
        /// </summary>
        public ILog Logger
        {
            get
            {
                if (_logger == null)
                {
                    lock (_loggerLock)
                    {
                        _logger = LogManager.GetLogger(typeof(Keyring));
                    }
                }
                return _logger;
            }
            set
            {
                _logger = value;
            }
        }

        #endregion

        #region Implementation of IDataProtector

        /// <summary>
        /// Protects the userData parameter and returns a string.
        /// </summary>
        /// <param name="userData">A string containing data to protect. </param>
        /// <param name="optionalEntropy">An additional byte array used to encrypt the data. </param>
        /// <param name="scope">One of the DataProtectionScope values.</param>
        /// <returns>A string representing the encrypted data.</returns>
        public byte[] Protect(string userData, byte[] optionalEntropy, DataProtectionScope scope)
        {
            // we do not use [DebugLogged] because it will compromise the security of the secrets in the logs
            Logger.DebugFormat("> ProtectData(\"****\", {0}, {1})", optionalEntropy, scope);
            if(userData != null)
            {
                var userDataBytes = Encoding.UTF8.GetBytes(userData);
                var encryptedBytes = ProtectedData.Protect(userDataBytes, optionalEntropy, scope);

                return encryptedBytes;
            }
            Logger.Info("User data is NULL");
            return null;
        }

        /// <summary>
        /// Unprotects the encryptedData parameter and returns a string.
        /// </summary>
        /// <param name="encryptedData">A string containing data encrypted using the Protect method.</param>
        /// <param name="optionalEntropy">An additional byte array used to encrypt the data.</param>
        /// <param name="scope">One of the DataProtectionScope values. </param>
        /// <returns>A string representing the unprotected data.</returns>
        public string Unprotect(byte[] encryptedData, byte[] optionalEntropy, DataProtectionScope scope)
        {
            // we do not use [DebugLogged] because it will compromise the security of the secrets in the logs
            Logger.DebugFormat("> Unprotect({0}, {1}, {2})", encryptedData, optionalEntropy, scope);
            if (encryptedData != null)
            {
                var userDataBytes = ProtectedData.Unprotect(encryptedData, optionalEntropy, scope);
                return Encoding.UTF8.GetString(userDataBytes);
            }
            Logger.Info("Encrypted data is NULL");
            return null;
        }

        #endregion
    }
}
