/* Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Pea <manuel.delapena@canonical.com>
 */
using System;
using System.Collections.Generic;
using System.Security.Cryptography;

namespace Canonical.Ubuntu.SSO
{
    public interface IOAuth
    {
        /// <summary>
        /// Generate the signature base that is used to produce the signature
        /// </summary>
        /// <param name="url">The full url that needs to be signed including its non OAuth url parameters</param>
        /// <param name="consumerKey">The consumer key</param>        
        /// <param name="token">The token, if available. If not available pass null or an empty string</param>
        /// <param name="tokenSecret">The token secret, if available. If not available pass null or an empty string</param>
        /// <param name="httpMethod">The http method used. Must be a valid HTTP method verb (POST,GET,PUT, etc)</param>
        /// <param name="nonce"></param>
        /// <param name="signatureType">The signature type. To use the default values use <see cref="OAuth.SignatureTypes">OAuth.SignatureTypes</see>.</param>
        /// <param name="timeStamp"></param>
        /// <param name="normalizedUrl"></param>
        /// <param name="normalizedRequestParameters"></param>
        /// <returns>The signature base</returns>
        string GenerateSignatureBase(Uri url, string consumerKey, string token, string tokenSecret, 
            string httpMethod, string timeStamp, string nonce, string signatureType, out string normalizedUrl, 
            out string normalizedRequestParameters);

        /// <summary>
        /// Generate the signature value based on the given signature base and hash algorithm
        /// </summary>
        /// <param name="signatureBase">The signature based as produced by the GenerateSignatureBase method or by any other means</param>
        /// <param name="hash">The hash algorithm used to perform the hashing. If the hashing algorithm requires initialization or a key it should be set prior to calling this method</param>
        /// <returns>A base64 string of the hash value</returns>
        string GenerateSignatureUsingHash(string signatureBase, HashAlgorithm hash);

        /// <summary>
        /// Generates a signature using the HMAC-SHA1 algorithm
        /// </summary>		
        /// <param name="url">The full url that needs to be signed including its non OAuth url parameters</param>
        /// <param name="consumerKey">The consumer key</param>
        /// <param name="consumerSecret">The consumer seceret</param>
        /// <param name="token">The token, if available. If not available pass null or an empty string</param>
        /// <param name="tokenSecret">The token secret, if available. If not available pass null or an empty string</param>
        /// <param name="httpMethod">The http method used. Must be a valid HTTP method verb (POST,GET,PUT, etc)</param>
        /// <param name="timeStamp"></param>
        /// <param name="nonce"></param>
        /// <param name="normalizedUrl"></param>
        /// <param name="normalizedRequestParameters"></param>
        /// <returns>A base64 string of the hash value</returns>
        string GenerateSignature(Uri url, string consumerKey, string consumerSecret, string token, 
            string tokenSecret, string httpMethod, string timeStamp, string nonce, out string normalizedUrl, 
            out string normalizedRequestParameters);

        /// <summary>
        /// Generates a signature using the specified signatureType 
        /// </summary>		
        /// <param name="url">The full url that needs to be signed including its non OAuth url parameters</param>
        /// <param name="consumerKey">The consumer key</param>
        /// <param name="consumerSecret">The consumer seceret</param>
        /// <param name="token">The token, if available. If not available pass null or an empty string</param>
        /// <param name="tokenSecret">The token secret, if available. If not available pass null or an empty string</param>
        /// <param name="httpMethod">The http method used. Must be a valid HTTP method verb (POST,GET,PUT, etc)</param>
        /// <param name="nonce"></param>
        /// <param name="signatureType">The type of signature to use</param>
        /// <param name="timeStamp"></param>
        /// <param name="normalizedUrl"></param>
        /// <param name="normalizedRequestParameters"></param>
        /// <returns>A base64 string of the hash value</returns>
        string GenerateSignature(Uri url, string consumerKey, string consumerSecret, string token, 
            string tokenSecret, string httpMethod, string timeStamp, string nonce, 
            OAuth.SignatureTypes signatureType, out string normalizedUrl, out string normalizedRequestParameters);

        /// <summary>
        /// Returns a key value pair that contains a hedaer that can be used to authenticate a web requet.
        /// </summary>
        /// <param name="url">The uri with the web resource.</param>
        /// <param name="realm">The real that was used for the oauth.</param>
        /// <param name="consumerKey">The oauth consumer key.</param>
        /// <param name="consumerSecret">The oauth consumer secret.</param>
        /// <param name="token">The oauth token.</param>
        /// <param name="tokenSecret">The aoauth token secret.</param>
        /// <param name="httpMethod">The http method used for the request.</param>
        /// <param name="timeStamp">A time stamp.</param>
        /// <param name="nonce">A once</param>
        /// <param name="signatureType">Teht ype of signature that will be used.</param>
        /// <param name="normUrl"></param>
        /// <param name="normParams"></param>
        /// <returns>A key value pair where the key is the name of the header and the value is the data of the header.</returns>
        KeyValuePair<string, string> GenerateHeaderWithSignature(Uri url, string realm, string consumerKey, 
            string consumerSecret, string token, string tokenSecret, string httpMethod, string timeStamp, 
            string nonce, OAuth.SignatureTypes signatureType, out string normUrl, out string normParams);

        /// <summary>
        /// Generate the timestamp for the signature        
        /// </summary>
        /// <returns></returns>
        string GenerateTimeStamp();

        /// <summary>
        /// Generate a nonce
        /// </summary>
        /// <returns></returns>
        string GenerateNonce();
    }
}