﻿/*
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using Canonical.UbuntuOne.Common.Aop;
using Canonical.UbuntuOne.Common.Validation;
using Canonical.UbuntuOne.Common.Win32;
using log4net;
using Microsoft.Win32;

namespace Canonical.Ubuntu.SSO
{
    /// <summary>
    /// Implementation of the IKeyring interface that will store encripted secrets in the users current keyring.
    /// </summary>
    public class Keyring : IKeyring
    {
        #region Variables

        internal const string RootPath = "Canonical\\Keyrings";
        internal const string Entropy = "bdc97980-bbfa-11df-851a-0800200c9a66";
        private const int KeyringMaxNameLength = 255;
        private ILog _logger;
        private readonly object _loggerLock = new object();
        private bool _keyringsRootExists;

        #endregion

        #region DI properties

        /// <summary>
        /// Gets and sets the users registry key.
        /// </summary>
        public IRegistryKey UserRegistry { get; set; }

        /// <summary>
        /// Gets and sets the data protector to secure the data in the registry.
        /// </summary>
        public IDataProtector DataProtector { get; set; }

        #endregion

        #region Properties

        /// <summary>
        /// Gets and sets the logger that will be used to log the operation of the
        /// class.
        /// </summary>
        public ILog Logger
        {
            get
            {
                if (_logger == null)
                {
                    lock (_loggerLock)
                    {
                        _logger = LogManager.GetLogger(typeof(Keyring));
                    }
                }
                return _logger;
            }
            set
            {
                _logger = value;
            }
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Creates a new instance of the class that will be using the users registry to store the data.
        /// </summary>
        public Keyring()
            : this(new RegistryKeyWrapper())
        {
            
        }

        /// <summary>
        /// Creates a new instance of the class that will be storing the keyring in the provided registry key.
        /// </summary>
        /// <param name="key"></param>
        public Keyring(IRegistryKey key)
        {
            UserRegistry = key;
        }

        #endregion

        #region Helpers

        /// <summary>
        /// Determines if the root used to store the keyrings does exist in the current user
        /// registry key. 
        /// </summary>
        /// <returns>A boolean flag that states if the root of the keyrings exists.</returns>
        [DebugLogged]
        private bool KeyringsRootExists()
        {
            if (!_keyringsRootExists)
            {
                var rootKey = from key in UserRegistry.GetSubKeyNames()
                              where key == "Canonical"
                              select key;
                if (rootKey.Count() != 1)
                {
                    _keyringsRootExists = false;
                    return _keyringsRootExists;
                }
                
                using (var canonicalKey = UserRegistry.OpenSubKey("Canonical"))
                {
                    rootKey = from key in canonicalKey.GetSubKeyNames()
                              where key == "Keyrings"
                              select key;
                    _keyringsRootExists = rootKey.Count() == 1;
                }
            
            }
            return _keyringsRootExists;
        }

        /// <summary>
        /// States if a keyring with the given name exists in the ssytem.
        /// </summary>
        /// <param name="keyring">The name of the keyring we are looking for.</param>
        /// <returns>A flag stating if the keyring does exist.</returns>
        [DebugLogged]
        private bool KeyringExist(string keyring)
        {
            // open the root keyring and then the subkeyring to ensure that the
            // length of the string is not over 255 chars
            if (!KeyringsRootExists())
                return false;

            using (var rootKeyring = UserRegistry.OpenSubKey(RootPath))
            {
                var presentKeyrings = from key in rootKeyring.GetSubKeyNames()
                                      where key == keyring
                                      select key;
                return presentKeyrings.Count() == 1;
            }
        }

        /// <summary>
        /// Creates the root path used by the keyrings.
        /// </summary>
        /// <returns>The registry key that represents the root path of the keyrings.</returns>
        [DebugLogged]
        private IRegistryKey CreateKeyringRootPath()
        {
            // create a key with the root path to be used
            return UserRegistry.CreateSubKey(RootPath);
        }

        /// <summary>
        /// Creates a keyring with the given name.
        /// </summary>
        /// <param name="keyring">The registry key that is used to store the data in the keyring.</param>
        [DebugLogged]
        private void CreateKeyring(string keyring)
        {
            using(var rootKey = (KeyringsRootExists())?
                UserRegistry.OpenSubKey(RootPath) : CreateKeyringRootPath())
            {
                var subkey = rootKey.CreateSubKey(keyring);
                subkey.Dispose();
            }
        }

        /// <summary>
        /// Allows to open a keyring with the given name.
        /// </summary>
        /// <param name="keyring">The name of the keyring to open.</param>
        /// <returns>A keyring key that is used to manage the keyring.</returns>
        [DebugLogged]
        private IRegistryKey OpenKeyring(string keyring)
        {
            using(var rootKey = UserRegistry.OpenSubKey(RootPath,RegistryKeyPermissionCheck.ReadWriteSubTree))
            {
                return rootKey.OpenSubKey(keyring, RegistryKeyPermissionCheck.ReadWriteSubTree);
            }
        }

        /// <summary>
        /// Allows to save a value in the keyring with the given name.
        /// </summary>
        /// <param name="keyringName">The name of the keyring in which the secret will be stored.</param>
        /// <param name="applicationName">The name of the application that is storing the secret.</param>
        /// <param name="secret">The secret to be stored.</param>
        private void SaveValue(string keyringName, string applicationName, string secret)
        {
            // we do not use the [DebugLogged] because we will be printing the secret and therefore
            // expose private data, instead with used the normal logging
            Logger.DebugFormat("> SaveValue({0}, {1}, \"******\")", keyringName, applicationName);
            using(var keyring = OpenKeyring(keyringName))
            {
                var encryptedSecret = DataProtector.Protect(secret, Encoding.UTF8.GetBytes(Entropy),
                    DataProtectionScope.CurrentUser);
                keyring.SetValue(applicationName, encryptedSecret);
            }
        }

        /// <summary>
        /// Allows to retrieve a value from a given keyring.
        /// </summary>
        /// <param name="keyringName">The name of the keyring from which the value will be read.</param>
        /// <param name="applicationName">The anme of the application whose secret we want to read.</param>
        /// <returns>The secret that was read.</returns>
        private string GetValue(string keyringName, string applicationName)
        {
            // we do not user [DbugLogged] because it will be printing the secret and therefore
            // expose private data. insetead we use normal logging
            Logger.DebugFormat("> GetValue({0}, {1})", keyringName, applicationName);
            using (var keyring = OpenKeyring(keyringName))
            {
                var secret = keyring.GetValue(applicationName) as byte[];
                Logger.Debug("< GetValue(\"******\")");
                return DataProtector.Unprotect(secret, Encoding.UTF8.GetBytes(Entropy), DataProtectionScope.CurrentUser);
            }    
        }


        #endregion

        #region Implementation of IKeyring

        /// <summary>
        /// Creates a new secret in the the keyring with the given name for a given applciation.
        /// </summary>
        /// <param name="keyringName">The name of the keyring where the secret will be stored.</param>
        /// <param name="applicationName">The name of the application whose secret will be stored.</param>
        /// <param name="secret">The secret to store in the keyring.</param>
        [DebugLogged]
        public void CreateSecret(string keyringName, string applicationName, string secret)
        {
            ValidateArgs.Begin()
                .IsNotNullOrEmpty(keyringName, "keyringName")
                .IsShorterThan(keyringName, KeyringMaxNameLength, "keyringName")
                .IsNotNullOrEmpty(applicationName, "applicationName")
                .IsNotNullOrEmpty(secret, "secret")
                .Check();

            if (!KeyringExist(keyringName))
            {
                Logger.InfoFormat("Keyring '{0}' does not exist.", keyringName);
                CreateKeyring(keyringName);
                Logger.InfoFormat("Creating keyring with name {0}", keyringName);
            }
            SaveValue(keyringName, applicationName, secret);
        }

        /// <summary>
        /// Gets the secret froma keyring using the name of the application that stored it.
        /// </summary>
        /// <param name="keyringName">The name of the keyring where the secret was stored.</param>
        /// <param name="applicationName">the name of the application that stored the secret.</param>
        /// <returns>An empty string if the secret does not exist, the secret otherwhise.</returns>
        [DebugLogged]
        public string GetSecretByName(string keyringName, string applicationName)
        {
            ValidateArgs.Begin()
                .IsNotNullOrEmpty(keyringName, "keyringName")
                .IsShorterThan(keyringName, KeyringMaxNameLength, "keyringName")
                .IsNotNullOrEmpty(applicationName, "applicationName")
                .Check();

            return KeyringExist(keyringName) ? GetValue(keyringName, applicationName) : null;
        }

        /// <summary>
        /// Gets a lists with all the names of the available keyrings.
        /// </summary>
        /// <returns>An enumerable with all the names of the different keyrings.</returns>
        [DebugLogged]
        public IEnumerable<string> GetKeyrings()
        {
            if(KeyringsRootExists())
            {
                using(var keyringRoot =  UserRegistry.OpenSubKey(RootPath))
                {
                    return keyringRoot.GetSubKeyNames();
                }
            }
            Logger.Info("Keyrings root does not exist.");
            return new string[0];
        }

        /// <summary>
        /// Gets a list with all the applications in a keyring.
        /// </summary>
        /// <param name="keyringName">The name of the keyring that is queried.</param>
        /// <returns>An enumerable with all the applications in the keyring.</returns>
        [DebugLogged]
        public IEnumerable<string> GetApplications(string keyringName)
        {
            ValidateArgs.Begin()
                .IsNotNullOrEmpty(keyringName, "keyring")
                .IsShorterThan(keyringName, KeyringMaxNameLength, "keyring")
                .Check();
            if(KeyringExist(keyringName))
            {
                using(var keyringRoot = UserRegistry.OpenSubKey(RootPath))
                {
                    using(var keyring = keyringRoot.OpenSubKey(keyringName))
                    {
                        return keyring.GetSubKeyNames();
                    }
                }
            }
            Logger.InfoFormat("Keyring '{0}' does not exist.", keyringName);
            return new string[0];
        }

        #endregion

        #region Implementation of IDisposable

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        /// <filterpriority>2</filterpriority>
        public void Dispose()
        {
            if (UserRegistry != null)
                UserRegistry.Dispose();
        }

        #endregion
    }
}
