﻿/*
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of Ubuntu One on Windows.
 * 
 * Ubuntu One on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with Ubuntu One for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System;
using System.Collections.Generic;
using System.Windows;
using Canonical.UbuntuOne.Common.Aop;

namespace Canonical.Ubuntu.SSO
{
    /// <summary>
    /// Implementation of the SSOCredentials provider that uses the DAPI framework to encrypt the
    /// data and store it in the registry.
    /// </summary>
    public class SSOCredentialsProvider : ISSOCredentialsProvider
    {
        private const string ApplicationTokenName = "Ubuntu One @ {0} (Windows)";
        internal const string KeyringName = "Default";
        internal const string ApplicationName = "UbuntuOne";

        #region DI properties

        /// <summary>
        /// Gets and sets the view that is used to tell the user to provide his login details.
        /// </summary>
        public ILoginView LoginView { get; set; }

        /// <summary>
        /// Gets and sets the processor that is used to login the user to the SSO service.
        /// </summary>
        public ISSOLoginProcessor SSOLoginProcessor { get; set; }

        /// <summary>
        /// Gets and sets the keyring that will be used to store the secrets.
        /// </summary>
        public IKeyring Keyring { get; set; }

        #endregion

        #region Implementation of ISSOCredentialsProvider

        public event EventHandler<CredentialsFoundEventArgs> OnCredentialsFound;
        public event EventHandler<CredentialsDeniedEventArgs> OnCredetialsDenied;
        public event EventHandler<CredentialsErrorEventArgs> OnCredentialsError;

        /// <summary>
        /// This method will try to fetch the encrypted credentials from the registry and 
        /// return them, and if the credentials are not found it will open a 
        /// dialog letting the user just log into an existing account via the 
        /// Ubuntu SSO webservice, and will then get the related credentials, 
        /// store them into the keyring for future usage and finally return them.
        /// 
        /// The result of this action will be notified through the use of the event handlers
        /// due to the fact that the call to the sso webservice is async.
        /// </summary>
        [DebugLogged]
        public void LoginToGetCredentials(string eventName, string applicationName)
        {
            try
            {
                // try to get the credentials from the keyring
                var secret = Keyring.GetSecretByName(KeyringName, ApplicationName);
                if (secret == null)
                {
                    // we need to use the processor to get the login info
                    if (LoginView.IsShown)
                        return;

                    if(LoginView.ShowDialog() != MessageBoxResult.Cancel)
                    {
                        var tokenName = string.Format(ApplicationTokenName, Environment.MachineName);
                        secret = SSOLoginProcessor.Login(LoginView.EmailAddress, LoginView.Password, tokenName);
                        // save the credentials in the keyring
                        // TODO: We have an issue here since we have more than one listening for the same credntials of Ubuntu One
                        Keyring.CreateSecret(KeyringName, applicationName, secret);
                    }
                    else
                    {
                        if (OnCredetialsDenied != null)
                        {
                            OnCredetialsDenied(this, new CredentialsDeniedEventArgs(eventName));
                        }
                        return;
                    }
                    
                }
                // we execute the credentials found event
                if(!string.IsNullOrEmpty(secret) && OnCredentialsFound != null)
                {
                    string token;
                    string tokenSecret;
                    string consumerKey;
                    string consumerSecret;
                    SSOLoginProcessor.SSOCredentialsEncoder.Decode(secret, out token, out tokenSecret, out consumerKey, out consumerSecret);
                    var credentials = new Dictionary<string, string>
                    {
                        {"Token", token},
                        {"TokenSecret", tokenSecret},
                        {"ConsumerKey", consumerKey},
                        {"ConsumerSecret", consumerSecret}
                    };
                    OnCredentialsFound(this, new CredentialsFoundEventArgs(eventName, credentials));
                }
            }
            catch (SSOLoginException e)
            {
                if (OnCredetialsDenied != null)
                {
                    OnCredetialsDenied(this, new CredentialsDeniedEventArgs(eventName));
                }
            }
            catch (Exception e)
            {
                // we have an error, therefore we will execute the OnCredentialsError
                if (OnCredentialsError != null)
                {
                    OnCredentialsError(this, new CredentialsErrorEventArgs(eventName, e.Message, e.StackTrace));
                }
            }


        }

        /// <summary>
        /// This method will try to fetch the credentials from the registry and return them, 
        /// and if the credentials are not found it will open a dialog letting the user 
        /// either register a new or log into an existing account via the Ubuntu SSO webservice, 
        /// and will then get the related credentials, store them into the registry for future 
        /// usage and finally return them.
        /// 
        /// The result of this action will be notified through the use of the event handlers
        /// due to the fact that the call to the sso webservice is async.
        /// </summary>
        [DebugLogged]
        public void LoginOrRegisterToGetCredentials()
        {
            throw new NotImplementedException();
        }

        #endregion
    }
}
