﻿/*
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of Ubuntu One on Windows.
 * 
 * Ubuntu One on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with Ubuntu One for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System;
using System.Net;
using Canonical.Ubuntu.SSO.Service;
using Canonical.UbuntuOne.Common.Net;
using Newtonsoft.Json.Linq;

namespace Canonical.Ubuntu.SSO
{
    /// <summary>
    /// Implementation of the ISSOLoginProcessor that uses the Ubuntu SSO services to perform
    /// the operations.
    /// </summary>
    public class SSOLoginProcessor : ISSOLoginProcessor
    {
        private const string ApplicationTokenName = "Ubuntu One @ {0} (Windows)";
        internal const string KeyringName = "Default";
        internal const string ApplicationName = "UbuntuOne";

        #region DI properties

        /// <summary>
        /// Gets and sets the Authentications rest service that is used to authenticate the user.
        /// </summary>
        public Authentications Authentications { get; set; }

        /// <summary>
        /// Gets and sets the keyring that will be used to store the secrets.
        /// </summary>
        public IKeyring Keyring { get; set; }

        /// <summary>
        /// Gets and sets the credential encoder that will be used to encode the oauth credentials 
        /// before they are saved in the keyring.
        /// </summary>
        public ISSOCredentialsEncoder SSOCredentialsEncoder { get; set; }

        /// <summary>
        /// Gets and sets the object that will take care of the oauth operations.
        /// </summary>
        public IOAuth OAuth { get; set; }

        public IHttpWebRequestFactory HttpWebRequestFactory { get; set; }

        #endregion

        #region Helper methods}

        /// <summary>
        /// Creates an oauth request that can access a protected web resource.
        /// </summary>
        /// <param name="uri">The uri where the resource is found.</param>
        /// <param name="httpMethod">The request berb to be used.</param>
        /// <param name="consumerKey">Consumer key from oauth.</param>
        /// <param name="consumerSecret">Consumer secret from oauth.</param>
        /// <param name="token">Token from oauth.</param>
        /// <param name="tokenSecret">Token secret from oauth.</param>
        /// <returns>A webrequest that can be used to access the resource.</returns>
        public IHttpWebRequest MakeRequest(string uri, string httpMethod, string consumerKey, string consumerSecret, string token, string tokenSecret)
        {
            // Form the full REST request url
            var url = new Uri(uri);
            string normUrl;
            string normParams;

            // get oauth header
            var authHeader = OAuth.GenerateHeaderWithSignature(url, string.Empty, consumerKey, consumerSecret,
                token, tokenSecret, httpMethod, OAuth.GenerateTimeStamp(), OAuth.GenerateNonce(), SSO.OAuth.SignatureTypes.HMACSHA1,
                out normUrl, out normParams);

            var request = HttpWebRequestFactory.Create(url);
            request.Headers.Add(authHeader.Key, authHeader.Value);
            return request;
        }

        /// <summary>
        /// Pings the Ubuntu One server so that the SSO tokens are added to them and they know how to identify the user.
        /// </summary>
        /// <param name="email">The email used by the user to register to u1.</param>
        /// <param name="consumerKey">The oauth consumer key.</param>
        /// <param name="consumerSecret">The oauth consumer secret.</param>
        /// <param name="token">The oauth token.</param>
        /// <param name="tokenSecret">The oauth secret.</param>
        private void PingUbuntuOneServer(string email, string consumerKey, string consumerSecret, string token, string tokenSecret)
        {
            // ping the service to make it download the tokens
            var pingRequest = MakeRequest(Constants.PingUrl + email, "GET", consumerKey, consumerSecret,
                                          token, tokenSecret);
            try
            {
                pingRequest.GetResponse();
            }
            catch (Exception e)
            {
                throw new SSOException("Credentials could not be sent to Ubuntu One servers", e);
            }
        }

        /// <summary>
        /// Allows to parse the json that is returned by the rest service to extract that credentials.
        /// </summary>
        /// <param name="credentialsJson">Json that contains the credentials info.</param>
        /// <param name="consumerKey">Out var where the oauth consumer key will be returned.</param>
        /// <param name="consumerSecret">Out var where the oauth consumer secret will be returned.</param>
        /// <param name="token">Out var where the oauth token secret will be returned.</param>
        /// <param name="tokenSecret">Out var where the oauth token secret will be returned.</param>
        private static void ParseCredentials(string credentialsJson, out string consumerKey, out string consumerSecret, out string token,
            out string tokenSecret)
        {
            var credentials = JObject.Parse(credentialsJson);
            try
            {
                consumerKey = (string)credentials["consumer_key"];
                consumerSecret = (string)credentials["consumer_secret"];
                token = (string)credentials["token"];
                tokenSecret = (string)credentials["token_secret"];

            }
            catch (InvalidCastException e)
            {
                // could happen when the json is not formatted as we expected.
                throw new SSOLoginException("Error when retrieving credentials.", e);
            }
            if (string.IsNullOrEmpty(consumerKey)
               || string.IsNullOrEmpty(consumerSecret)
               || string.IsNullOrEmpty(token)
               || string.IsNullOrEmpty(tokenSecret))
            {
                // none of them can be null or empty.
                throw new SSOLoginException("Credentials have missing values..");
            }

        }
        #endregion

        #region Implementation of ISSOLoginProcessor

        /// <summary>
        /// Generates a new captha and stores it in the passed file path.
        /// </summary>
        /// <param name="filePath">The path in which the captcha should be stored.</param>
        /// <returns></returns>
        public string GenerateCaptcha(string filePath)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Registers a new user to the system.
        /// </summary>
        /// <param name="email">The email of the new user to register.</param>
        /// <param name="password">The password that the new user will use to login.</param>
        /// <param name="captchaId">The id of the captcha that the user used to register.</param>
        /// <param name="captchaSolution">The solution given by the user for the given captcha</param>
        /// <returns>The email of the new user if there were no login issues.</returns>
        public string RegisterUser(string email, string password, string captchaId, string captchaSolution)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Allows a user to login using the ubuntu sso.
        /// </summary>
        /// <param name="email">The email of the user.</param>
        /// <param name="password">The password to be used for login.</param>
        /// <param name="tokenName">The token name used for login.</param>
        /// <returns>A string with the sso credentials.</returns>
        public string Login(string email, string password, string tokenName)
        {
            
            try
            {
                // credentials are not present in the keyring, therefore we will have to get them from the 
                // rest service
                string token;
                string tokenSecret;
                string consumerKey;
                string consumerSecret;

                // the rest server returns json, we need to parse it.
                ParseCredentials(Authentications.Authenticate(email, password, tokenName),
                    out consumerKey, out consumerSecret, out token, out tokenSecret);
                var secret = SSOCredentialsEncoder.Encode(token, tokenSecret, consumerKey, consumerSecret);
                // ping the service to make it download the tokens
                PingUbuntuOneServer(email, consumerKey, consumerSecret, token, tokenSecret);
                return secret;
            }
            catch (WebException e)
            {
                throw new SSOLoginException("Unable to login", e);
            }
            catch (Exception e)
            {
                throw new SSOLoginException("Unable to login", e);
            }
        }

        /// <summary>
        /// Validate an email token for a user.
        /// </summary>
        /// <param name="email">The email address of the user.</param>
        /// <param name="password">The apssword of the user.</param>
        /// <param name="emailToken">Teh email token to validate.</param>
        /// <param name="tokenName">The name of the token to validate.</param>
        /// <returns>The token returned by the server.</returns>
        public string ValidateEmail(string email, string password, string emailToken, string tokenName)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Allwos to request a password reset.
        /// </summary>
        /// <param name="email">The email of the user requesting the password reset.</param>
        public void RequestPasswordResetToken(string email)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Set a new password for the account.
        /// <remarks>
        /// The 'token' has to be the one resulting from a call to 'RequestPasswordResetToken'.
        /// </remarks>
        /// </summary>
        /// <param name="email">The email of the user.</param>
        /// <param name="token">The token received from requesting the password reset.</param>
        /// <param name="newPassword">The new password to be used by the account.</param>
        /// <returns>The email of the account if everyhing went ok.</returns>
        public string SetNewPassword(string email, string token, string newPassword)
        {
            throw new NotImplementedException();
        }

        #endregion
    }
}
