﻿// Copyright 2010 Canonical Ltd.
// 
// This file is part of UbuntuOne on Windows.
// 
// UbuntuOne on Windows is free software: you can redistribute it and/or modify		
// it under the terms of the GNU Lesser General Public License version 		
// as published by the Free Software Foundation.		
//		
// Ubuntu One on Windows is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
// GNU Lesser General Public License for more details.	
//
// You should have received a copy of the GNU Lesser General Public License	
// along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
// 
// Authors: Manuel de la Peña <manuel.delapena@canonical.com>
using System;
using System.Collections.Generic;
using Canonical.UbuntuOne.Client.Notification;
using Canonical.UbuntuOne.Client.Preferences;
using Canonical.UbuntuOne.Common.Utils;
using log4net;
using NUnit.Framework;
using Rhino.Mocks;
using Rhino.Mocks.Interfaces;

namespace Canonical.UbuntuOne.Client.Test.Notification
{
    [TestFixture]
    public class NotificationPresenterFixture
    {

        #region Variables

        private MockRepository _mocks;
        private IApplication _app;
        private IExplorer _explorer;
        private IWebbrowser _browser;
        private IStateMapper _mapper;
        private IPreferencesDialogPresenter _preferences;
        private INotificationIconView _view;
        private ISyncDaemonClientNotifier _notifier;
        private NotificationIconPresenter _presenter;
        private ILog _logger;

        #endregion

        #region Setup

        [SetUp]
        public void Setup()
        {
            _mocks = new MockRepository();
            _app = _mocks.StrictMock<IApplication>();
            _explorer = _mocks.StrictMock<IExplorer>();
            _browser = _mocks.StrictMock<IWebbrowser>();
            _mapper = _mocks.StrictMock<IStateMapper>();
            _preferences = _mocks.StrictMock<IPreferencesDialogPresenter>();
            _view = _mocks.StrictMock<INotificationIconView>();
            _notifier = _mocks.DynamicMock<ISyncDaemonClientNotifier>();
            _logger = _mocks.StrictMock<ILog>();
            _presenter = new NotificationIconPresenter
            {
                Application = _app,
                Explorer = _explorer,
                Webbrowser = _browser,
                StateMapper = _mapper,
                PreferencesDialogPresenter = _preferences,
                NotificationIconView = _view,
                Logger = _logger
            };
            _mocks.BackToRecordAll();
        }

        #endregion

        #region Tests

        [Test]
        public void OnStateChange()
        {
            IEventRaiser onStateRaiser;
            var args = new StateEventArgs(StateEventArgs.StateEnum.OUT_OF_SYNC);
            using (_mocks.Record())
            {
                _notifier.OnStateChanged += null;
                onStateRaiser = LastCall.IgnoreArguments().GetEventRaiser();

                Expect.Call(_mapper.ToString(args.State))
                    .Return("")
                    .Repeat.Once();
                _view.State = "new state";

                LastCall.IgnoreArguments();
            }    
            using(_mocks.Playback())
            {
                _presenter.SyncDaemonClientNotifier = _notifier;
                onStateRaiser.Raise(this, args);
            }
        }

        [Test]
        public void OnRecentlyModifiedTest()
        {
            IEventRaiser onRecentlyModifiedRaiser;
            var args = new RecentlyModifiedEventArgs(new List<string>());
            using(_mocks.Record())
            {
                _notifier.OnRecentlyModifiedChanged += null;
                onRecentlyModifiedRaiser = LastCall.IgnoreArguments().GetEventRaiser();
                _view.RecentlyModifiedFiles = new List<string>();
                LastCall.IgnoreArguments();
            }
            using(_mocks.Playback())
            {
                _presenter.SyncDaemonClientNotifier = _notifier;
                onRecentlyModifiedRaiser.Raise(this, args);
            }
        }

        [Test]
        public void OnOperationCompletnessTest()
        {
            IEventRaiser onOperationCompletnessRaiser;
            var args = new OperationCompletenessEventArgs("dowload", 45, 23.4,
                OperationCompletenessEventArgs.SpeedUnitsEnum.KB);
            using(_mocks.Record())
            {
                _notifier.OnOperationCompletness += null;
                onOperationCompletnessRaiser = LastCall.IgnoreArguments().GetEventRaiser();
                _view.CurrentOperation = "";
                LastCall.IgnoreArguments();
            }
            using(_mocks.Playback())
            {
                _presenter.SyncDaemonClientNotifier = _notifier;
                onOperationCompletnessRaiser.Raise(this, args);
            }
        }

        [Test]
        public void OpenPreferencesTest()
        {
            using(_mocks.Record())
            {
                Expect.Call(() => _preferences.Show())
                    .Repeat.Once();
            }
            using(_mocks.Playback())
            {
                _presenter.OpenUbuntuOnePreferences();
            }
        }

        [Test]
        public void OpenShareLocationTest()
        {
            using(_mocks.Record())
            {
                Expect.Call(() => _explorer.OpenFolder(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments)))
                    .IgnoreArguments()
                    .Repeat.Once();
            }
            using(_mocks.Playback())
            {
                _presenter.OpenSharesLocation();
            }

        }

        [Test]
        public void OpenSyncsLocationTest()
        {
            using (_mocks.Record())
            {
                Expect.Call(() => _explorer.OpenFolder(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments)))
                    .IgnoreArguments()
                    .Repeat.Once();
            }
            using (_mocks.Playback())
            {
                _presenter.OpenSyncsLocation(); 
            }
        }

        [Test]
        public void OpenHelpTest()
        {
            using (_mocks.Record())
            {
                Expect.Call(() => _browser.LauncWebpage(NotificationIconPresenter.HelpUrl))
                    .Repeat.Once();
            }
            using (_mocks.Playback())
            {
                _presenter.OpenHelp();
            }
        }

        [Test]
        public void OpenUpgradeOptionTest()
        {
            using (_mocks.Record())
            {
                Expect.Call(() => _browser.LauncWebpage(NotificationIconPresenter.UpgradeUrl))
                    .Repeat.Once();
            }
            using (_mocks.Playback())
            {
                _presenter.OpenUpgradeOption();
            }
        }

        [Test]
        public void ExitApplicationTest()
        {
            using (_mocks.Record())
            {
                Expect.Call(() => _app.Exit())
                    .Repeat.Once();
            }
            using (_mocks.Playback())
            {
                _presenter.Exit();
            }
        }

        #endregion

    }
}
