﻿/* Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System;
using System.IO;
using System.ServiceModel;
using Canonical.Ubuntu.SSO;
using Canonical.UbuntuOne.Client.Preferences;
using Canonical.UbuntuOne.Common;
using Canonical.UbuntuOne.Common.Aop;
using Canonical.UbuntuOne.Common.Net;
using Canonical.UbuntuOne.Common.Utils;
using log4net;

namespace Canonical.UbuntuOne.Client.Notification
{
    /// <summary>
    /// Implementation of the INotificationIconPresenter interface that manages a view that is used
    /// to display notifications to the user.
    /// </summary>
    public class NotificationIconPresenter : INotificationIconPresenter
    {
        #region Variables

        /// <summary>
        /// The url to ebe openened when the user wants to upgrade.
        /// </summary>
        public const string UpgradeUrl = "https://one.ubuntu.com/upgrade";

        /// <summary>
        /// The user to be openened when the user requires help.
        /// </summary>
        public const string HelpUrl = "https://one.ubuntu.com/support/";

        private static readonly string SharesLocation = "";
        private static readonly string SyncsLocation = "";
        private ISyncDaemonClientNotifier _clientNotifier;
        private ISSOCredentialsProvider _ssoProvider;
        private ILog _logger;
        private readonly object _loggerLock = new object();
        private static string EventName = "UbuntuOneNotificationIconPresenter";
        private static string ApplicationName = "UbuntuOne";

        #endregion

        #region Di properties

        /// <summary>
        /// Gets and sets the dialog used to show the preferences.
        /// </summary>
        public IPreferencesDialogPresenter PreferencesDialogPresenter { get; set; }

        /// <summary>
        /// Gets and sets the views that will be used to show the user the information.
        /// </summary>
        public INotificationIconView NotificationIconView { get; set; }

        /// <summary>
        /// Gets and sets the mapper used to convert from the state enum to a string.
        /// </summary>
        public IStateMapper StateMapper { get; set; }

        /// <summary>
        /// Gets and sets the webbrowser to be used to show pages to the user.
        /// </summary>
        public IWebbrowser Webbrowser { get; set; }

        /// <summary>
        /// Get and sets the eplorer that will be used to open folder in the client 
        /// machine.
        /// </summary>
        public IExplorer Explorer { get; set; }

        /// <summary>
        /// Gets and sets the application that is running the notify presenter.
        /// </summary>
        public IApplication Application { get; set; }

        /// <summary>
        /// Gets and sets the message box used to send messages to the user.
        /// </summary>
        public IMessageBox MessageBox { get; set; }

        /// <summary>
        /// Gets and sets the provider that will return the credentials used in ubuntu one.
        /// </summary>
        public ISSOCredentialsProvider SSOCredentialsProvider
        {
            get { return _ssoProvider; }
            set
            {
                if (value == null) return;
                _ssoProvider = value;
                _ssoProvider.OnCredentialsFound += OnCredentialsFound;
                _ssoProvider.OnCredentialsError += OnCredentialsError;
                _ssoProvider.OnCredetialsDenied += OnCredentialsDenied;
            }
        }

        /// <summary>
        /// Gets and sets the notifier that will ensure that the presenter know about changes in the 
        /// sync daemon.
        /// </summary>
        public ISyncDaemonClientNotifier SyncDaemonClientNotifier
        {
            get { return _clientNotifier; }
            set
            {
                if (value == null) return;
                _clientNotifier = value;
                // register to the different events raised by the client
                _clientNotifier.OnNotification += OnNotificationHandler;
                _clientNotifier.OnOperationCompletness += OnOperationCompletnessHandler;
                _clientNotifier.OnRecentlyModifiedChanged += OnRecentlyModifiedChangedHandler;
                _clientNotifier.OnStateChanged += OnStateChangedHandler;
            }
        }

        /// <summary>
        /// Gets and sets the channel factory that can be used to create communication objects that are used to
        /// interact with the sync daemon.
        /// </summary>
        public ISyncDaemon SyncDaemon { get; set; }

        #endregion

        #region Properties

        /// <summary>
        /// Gets and sets the logger used by the class.
        /// </summary>
        internal ILog Logger
        {
            get
            {
                if (_logger == null)
                {
                    lock (_loggerLock)
                    {
                        _logger = LogManager.GetLogger(typeof(NotificationIconPresenter));
                    }
                }
                return _logger;
            }
            set { _logger = value; }
        }

        #endregion

        #region Constructors

        static NotificationIconPresenter()
        {
            SharesLocation = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            SyncsLocation = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments), "UbuntuOne");

        }

        #endregion

        #region Helper methods

        /// <summary>
        /// Helper method that will ensure that when the state of the sync daemon changes this change
        /// is correctly reflected in the View.
        /// </summary>
        /// <param name="sender">The object that sent the event.</param>
        /// <param name="e">The arguments that describe the state change.</param>
        [DebugLogged]
        private void OnStateChangedHandler(object sender, StateEventArgs e)
        {
            NotificationIconView.State = StateMapper.ToString(e.State);
        }

        /// <summary>
        /// Helper method that will ensure that when the recently changed files are updated by the daemon that 
        /// such a change is correcty displayed in the view.
        /// </summary>
        /// <param name="sender">The object that sent the event.</param>
        /// <param name="e">The arguments that describe the changes in the recently changed list.</param>
        [DebugLogged]
        private void OnRecentlyModifiedChangedHandler(object sender, RecentlyModifiedEventArgs e)
        {
            NotificationIconView.RecentlyModifiedFiles = e.RecentlyModifiedFiles;
        }

        /// <summary>
        /// Helper method that will ensure that when the status of an operation that is being performed by the daemon
        /// changes that such a changed ins correctly reflected in the UI.
        /// </summary>
        /// <param name="sender">The object that sent the event.</param>
        /// <param name="e">The argumets that describe the new updated operation state.</param>
        [DebugLogged]
        private void OnOperationCompletnessHandler(object sender, OperationCompletenessEventArgs e)
        {
            NotificationIconView.CurrentOperation = String.Format(NotificationResources.OperationStateNotification,
                                                                  e.Operation, e.Percentage, e.Speed, e.SpeedUnits);
        }

        /// <summary>
        /// Helper method that will ensure that when a notification is sent by the daemon that it is correctly displayed in the UI.
        /// </summary>
        /// <param name="sender">The object that sent the event.</param>
        /// <param name="e">The arguments that contains the notification to display.</param>
        [DebugLogged]
        private void OnNotificationHandler(object sender, NotificationEventArgs e)
        {
            // TODO: Implement this method 
        }

        /// <summary>
        /// Helper method executed when the provider did return new credentials.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        [DebugLogged]
        private void OnCredentialsFound(object sender, CredentialsFoundEventArgs args)
        {
            // check if the credentials are for us
            if (args.ApplicationName != EventName) return;
            NotificationIconView.ViewDispatcher.Dispatch(
                () =>
                    {
                        NotificationIconView.IsManualSyncEnabled = false;
                        NotificationIconView.State = "Syncing";
                    }
            );
            
            Logger.Info("Got the UbuntuOne credentials!");
            var oneDir = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments),
                                      "UbuntuOne");
            // ensure that the path can be handled by the python code
            oneDir = oneDir.Replace("\\", "\\\\");
            SyncDaemon.ManualSync(
                oneDir,
                args.Credentials["Token"],
                args.Credentials["TokenSecret"],
                args.Credentials["ConsumerKey"],
                args.Credentials["ConsumerSecret"]);
            NotificationIconView.ViewDispatcher.Dispatch(
                () =>
                {
                    NotificationIconView.IsManualSyncEnabled = true;
                    NotificationIconView.State = "Synced";
                }
            );
        }

        [DebugLogged]
        private void OnCredentialsDenied(object sender, CredentialsDeniedEventArgs args)
        {

            MessageBox.ShowWarning("You need to login to be able to sync your files.");
        }

        [DebugLogged]
        private void OnCredentialsError(object sender, CredentialsErrorEventArgs args)
        {
            MessageBox.ShowError("An error courred when trying to retrieve your credentials.");
        }

        #endregion

        #region Implementation of INotificationIconPresenter

        /// <summary>
        /// When implemented this method should for the sync of the ubuntu one folder.
        /// </summary>
        public void ManualSync()
        {

            // we get the credentials, because we have register an action to the credetials aquired, we do nothing besides this
            SSOCredentialsProvider.LoginToGetCredentials(EventName, ApplicationName);
        }

        /// <summary>
        /// Opens the Preferences window of Ubuntu One.
        /// </summary>
        public void OpenUbuntuOnePreferences()
        {
            PreferencesDialogPresenter.Show();
        }

        /// <summary>
        /// Opens the explorer with the location of the shares of Ubuntu One.
        /// </summary>
        public void OpenSharesLocation()
        {
            try
            {
                Explorer.OpenFolder(SharesLocation);
            }
            catch (ExplorerException exception)
            {
                Logger.WarnFormat("There was an error operning the SharesLocation, {0}", exception);
                // use the view to notify the error
                NotificationIconView.ShowNotification(10, "Error", String.Format(NotificationResources.ErrorOpeningFolder,
                    Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments)));
                throw;
            }
        }

        /// <summary>
        /// Opens the explorer with the location of the sync folders.
        /// </summary>
        public void OpenSyncsLocation()
        {
            try
            {
                Explorer.OpenFolder(SyncsLocation);
            }
            catch (ExplorerException exception)
            {
                Logger.WarnFormat("There was an error operning the SharesLocation, {0}", exception);
                // use the view to notify the error
                MessageBox.ShowError(String.Format(NotificationResources.ErrorOpeningFolder,
                    Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments)));
                throw;
            }
        }

        /// <summary>
        /// Opens the location used to provide help to the user.
        /// </summary>
        [DebugLogged]
        public void OpenHelp()
        {
            Webbrowser.LauncWebpage(HelpUrl);
        }

        /// <summary>
        /// Opens the location used to upgrade the account of the user.
        /// </summary>
        [DebugLogged]
        public void OpenUpgradeOption()
        {
            Webbrowser.LauncWebpage(UpgradeUrl);
        }

        /// <summary>
        /// Close the client and the daemon.
        /// </summary>
        [DebugLogged]
        public void Exit()
        {
            Application.Exit();
        }

        #endregion
    }
}
