﻿/*
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System;
using System.IO;
using Canonical.Ubuntu.SSO;
using Canonical.UbuntuOne.Common;
using Canonical.UbuntuOne.Common.Utils;
using log4net;

namespace Canonical.UbuntuOne.Client.Preferences
{
    /// <summary>
    /// Task that will ensure that the preferences are loaded at boot time.
    /// </summary>
    public class LoadPreferencesStartupTask : IStartupTask
    {
        #region Variables

        private ISSOCredentialsProvider _ssoProvider;
        private const string EventName = "PreferencesStartupLoad";
        private const string ApplicationName = "UbuntuOne";
        private ILog _logger;
        private readonly object _loggerLock = new object();

        #endregion

        #region DI properties

        /// <summary>
        /// Gets and sets the logger used by the class.
        /// </summary>
        internal ILog Logger
        {
            get
            {
                if (_logger == null)
                {
                    lock (_loggerLock)
                    {
                        _logger = LogManager.GetLogger(typeof(PreferencesDialogPresenter));
                    }
                }
                return _logger;
            }
            set { _logger = value; }
        }

        /// <summary>
        /// Gets and sets the provider that will return the credentials used in ubuntu one.
        /// </summary>
        public ISSOCredentialsProvider SSOCredentialsProvider
        {
            get { return _ssoProvider; }
            set
            {
                if (value == null) return;
                _ssoProvider = value;
                _ssoProvider.OnCredentialsFound += OnCredentialsFound;
                _ssoProvider.OnCredentialsError += OnCredentialsError;
                _ssoProvider.OnCredetialsDenied += OnCredentialsDenied;
            }
        }

        /// <summary>
        /// Gets and sets the preferences manager used to ensure that the user has its preferences
        /// stored.
        /// </summary>
        public IPreferencesManager PreferencesManager { get; set; }

        /// <summary>
        /// Gets the sync daemon that is used to perform the sync operations.
        /// </summary>
        public ISyncDaemon SyncDaemon { get; set; }

        /// <summary>
        /// Gets and sets the message box used to send messages to the user.
        /// </summary>
        public IMessageBox MessageBox { get; set; }

        #endregion

        #region Helpers

        /// <summary>
        /// Helper method executed when the provider did return new credentials.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        private void OnCredentialsFound(object sender, CredentialsFoundEventArgs args)
        {
            // check if the credentials are for us
            if (args.ApplicationName != EventName) return;
            Logger.Info("Got the UbuntuOne credentials!");
            var oneDir = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments),
                                      "UbuntuOne");
            // ensure that the path can be handled by the python code
            oneDir = oneDir.Replace("\\", "\\\\");
            // call the ISyncService to perform the manual sync

            SyncDaemon.EnableManualSyncAutoExecution(
                    oneDir,
                    args.Credentials["Token"],
                    args.Credentials["TokenSecret"],
                    args.Credentials["ConsumerKey"],
                    args.Credentials["ConsumerSecret"],
                    (int)PreferencesManager.AutoSyncFrequency);
        }

        private void OnCredentialsDenied(object sender, CredentialsDeniedEventArgs args)
        {

            MessageBox.ShowWarning("You need to login to be able to sync your files.");
        }

        private void OnCredentialsError(object sender, CredentialsErrorEventArgs args)
        {
            MessageBox.ShowError("An error courred when trying to retrieve your credentials.");
        }

        #endregion

        public void Exeucte()
        {
            if (PreferencesManager.IsAutoSyncEnable)
                // we get the credentials, because we have register an action to the credetials aquired, we do nothing besides this
                SSOCredentialsProvider.LoginToGetCredentials(EventName, ApplicationName);
        }
    }
}
