﻿/**
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System.Windows;
using Canonical.UbuntuOne.Common.Update;
using log4net;
using Microsoft.ApplicationBlocks.Updater;
using NUnit.Framework;
using DotUpdater;
using Rhino.Mocks;
using Updater = Canonical.UbuntuOne.Common.Update.Updater;


namespace Canonical.UbuntuOne.Common.Tests.Update
{
    [TestFixture]
    public class UpdaterFixture
    {
        #region Setup

        private ILog _logger;
        private MockRepository _mocks;
        private Updater _updater;
        private IDownloadingPresenter _presenter;
        private IDotUpdaterWrapper _dotUpdaterWrapper;
        private IMessageBox _messageBox;

        [SetUp]
        public void Setup()
        {
            
            _mocks = new MockRepository();
            _logger = _mocks.DynamicMock<ILog>();
            _mocks.DynamicMock<Logger>();
            _dotUpdaterWrapper = _mocks.DynamicMock<IDotUpdaterWrapper>();
            _presenter = _mocks.DynamicMock<IDownloadingPresenter>();
            _messageBox = _mocks.StrictMock<IMessageBox>();
            _updater = new Updater
            {
                Logger = _logger,
                DownloadingPresenter = _presenter,
                DotUpdater = _dotUpdaterWrapper,
                MessageBox = _messageBox
            };
        }

        
        #endregion

        #region Tests

        [Test]
        public void DisposeTest()
        {
            using(_mocks.Record())
            {
                Expect.Call(() => _dotUpdaterWrapper.Dispose())
                    .Repeat.Once();
            }
            using(_mocks.Playback())
            {
                using (_updater)
                {
                    // do something
                }
            }
        }

        [Test]
        public void UpdatesArePresentFalseTest()
        {
            using(_mocks.Record())
            {
                Expect.Call(_dotUpdaterWrapper.CheckForUpdates())
                    .Return(null)
                    .Repeat.Once();
            }

            using(_mocks.Playback())
            {
                Assert.IsFalse(_updater.UpdatesArePresent());
            }
        }

        [Test]
        public void UpdatesArePresentTrueTest()
        {
            using(_mocks.Record())
            {
                Expect.Call(_dotUpdaterWrapper.CheckForUpdates())
                    .Return(new UpdaterTask("", "wewe", new string[0], "code", new string[0]))
                    .Repeat.Once();
            }

            using (_mocks.Playback())
            {
                Assert.IsTrue(_updater.UpdatesArePresent());
            }
        }

        [Test]
        [ExpectedException(typeof(SelfUpdateException))]
        public void PerformUpdateNoCheckDoneTest()
        {
            using(_mocks.Record())
            {
                Expect.Call(_messageBox.ShowYesNo(UpdateResources.DownloadUpdatesQuestion, MessageBoxImage.Question))
                    .Return(MessageBoxResult.Yes);
            }
            using (_mocks.Playback())
            {
                _updater.PerformUpdate();
            }
        }

        [Test]
        public void PerformUpdateUserSaysNoTest()
        {
            var task = _mocks.DynamicMock<UpdaterTask>("", "wewe", new string[0], "code", new string[0]);
            using (_mocks.Record())
            {
                Expect.Call(_messageBox.ShowYesNo(UpdateResources.DownloadUpdatesQuestion, MessageBoxImage.Question))
                    .Return(MessageBoxResult.No);
                Expect.Call(_dotUpdaterWrapper.CheckForUpdates())
                    .Return(task)
                    .Repeat.Once();
                Expect.Call(() => _dotUpdaterWrapper.BeginDownload(task))
                    .Repeat.Never();
            }
            using (_mocks.Playback())
            {
                if (_updater.UpdatesArePresent())
                {
                    _updater.PerformUpdate();
                }
            }
        }

        [Test]
        [ExpectedException(typeof(SelfUpdateException))]
        public void PerformUpdateCheckDoneNoUpdateTest()
        {
            using (_mocks.Record())
            {
                Expect.Call(_messageBox.ShowYesNo(UpdateResources.DownloadUpdatesQuestion, MessageBoxImage.Question))
                    .Return(MessageBoxResult.Yes);
                Expect.Call(_dotUpdaterWrapper.CheckForUpdates())
                    .Return(null)
                    .Repeat.Once();
            }
            using (_mocks.Playback())
            {
                var areAvialable = _updater.UpdatesArePresent();
                Assert.IsFalse(areAvialable);
                _updater.PerformUpdate();
            }
        }

        [Test]
        public void PerformUpdateNotDownloadedNotDownloadingTest()
        {
            var task = _mocks.DynamicMock<UpdaterTask>("", "wewe", new string[0], "code", new string[0]);
            using(_mocks.Record())
            {
                Expect.Call(_messageBox.ShowYesNo(UpdateResources.DownloadUpdatesQuestion, MessageBoxImage.Question))
                    .Return(MessageBoxResult.Yes);

                Expect.Call(_dotUpdaterWrapper.CheckForUpdates())
                    .Return(task)
                    .Repeat.Once();

                Expect.Call(() => _dotUpdaterWrapper.BeginDownload(task))
                    .Repeat.Once();

                // ensure the handlers have been registered
                _dotUpdaterWrapper.OnDownloadProgress += _presenter.DownloadProgressHandler;

                _dotUpdaterWrapper.OnDownloadCompleted += _presenter.DownloadUpdateCompleted;
                _dotUpdaterWrapper.OnDownloadCompleted += null;
                LastCall.IgnoreArguments();

                _dotUpdaterWrapper.OnDownloadError += _presenter.DownloadErrorHandler;
                _dotUpdaterWrapper.OnDownloadError += null;
                LastCall.IgnoreArguments();

            }
            using (_mocks.Playback())
            {
                if(_updater.UpdatesArePresent())
                {
                    _updater.PerformUpdate();
                }
            }
        }

        [Test]
        [ExpectedException(typeof(SelfUpdateException))]
        public void PerformUpdateDownloadExceptionTest()
        {
            var task = _mocks.DynamicMock<UpdaterTask>("", "wewe", new string[0], "code", new string[0]);
            using (_mocks.Record())
            {
                Expect.Call(_messageBox.ShowYesNo(UpdateResources.DownloadUpdatesQuestion, MessageBoxImage.Question))
                    .Return(MessageBoxResult.Yes);
                Expect.Call(_dotUpdaterWrapper.CheckForUpdates())
                    .Return(task)
                    .Repeat.Once();

                Expect.Call(() => _dotUpdaterWrapper.BeginDownload(task))
                    .Throw(new UpdaterException(""));
            }
            using (_mocks.Playback())
            {
                if (_updater.UpdatesArePresent())
                {
                    _updater.PerformUpdate();
                }
            }
        }

        [Test]
        [ExpectedException(typeof(SelfUpdateException))]
        public void CheckForUpdatesExceptionTest()
        {
            using(_mocks.Record())
            {
                Expect.Call(_dotUpdaterWrapper.CheckForUpdates())
                    .Throw(new UpdaterException(""));
            }
            using(_mocks.Playback())
            {
                if (_updater.UpdatesArePresent())
                {
                    _updater.PerformUpdate();
                }
            }
        }

        #endregion
    }
}
