﻿/*
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System;
using System.IO;
using System.Net;
using System.Net.Cache;
using System.Net.Security;
using System.Security.Cryptography.X509Certificates;
using System.Security.Principal;
using RealHttpRequest = System.Net.HttpWebRequest;

namespace Canonical.UbuntuOne.Common.Net
{
    /// <summary>
    /// Wrapper around the System.Net.HttpWebRequest.
    /// </summary>
    internal class HttpWebRequest : IHttpWebRequest
    {
        #region Variables

        private RealHttpRequest _request;

        #endregion

        #region Constructors

        /// <summary>
        /// Internal constructor that can only be used by a httpweb request factory.
        /// </summary>
        /// <param name="httpWebRequest"></param>
        internal HttpWebRequest(RealHttpRequest httpWebRequest)
        {
            _request = httpWebRequest;
        }

        #endregion

        #region Implementation of IHttpWebRequest

        /// <summary>
        /// Gets and sets value of the Accept HTTP header. The default value is null.
        /// </summary>
        public string Accept
        {
            get { return _request.Accept; }
            set { _request.Accept = value; }
        }

        /// <summary>
        /// Gets and sets the uri that responds to the request.
        /// </summary>
        public Uri Address
        {
            get { return _request.Address; }
        }

        /// <summary>
        /// Gets and sets if the request should automatically follow redirection responses
        /// </summary>
        public bool AllowAutoRedirect
        {
            get { return _request.AllowAutoRedirect; }
            set { _request.AllowAutoRedirect = value; }
        }

        /// <summary>
        /// Enables buffering of the data sent to the Internet resource
        /// </summary>
        public bool AllowWriteStreamBuffering
        {
            get { return _request.AllowWriteStreamBuffering; }
            set { _request.AllowWriteStreamBuffering = value; }
        }

        /// <summary>
        /// Gets and sets the auhentication lavel used.
        /// </summary>
        public AuthenticationLevel AuthenticationLevel
        {
            get { return _request.AuthenticationLevel; }
            set { _request.AuthenticationLevel = value; }
        }

        /// <summary>
        /// Gets and sets the type of decompression that is used. 
        /// </summary>
        public DecompressionMethods AutomaticDecompression
        {
            get { return _request.AutomaticDecompression; }
            set { _request.AutomaticDecompression = value; }
        }

        /// <summary>
        /// Gets or sets the cache policy used by the request.
        /// </summary>
        public RequestCachePolicy CachePolicy
        {
            get { return _request.CachePolicy; }
            set { _request.CachePolicy = value; }
        }

        /// <summary>
        /// Gets and sets the certificates used by the request.
        /// </summary>
        public X509CertificateCollection ClientCertificates
        {
            get { return _request.ClientCertificates; }
            set { _request.ClientCertificates = value; }
        }

        /// <summary>
        /// Gets and sets the Connection HTTP header
        /// </summary>
        public string Connection
        {
            get { return _request.Connection; }
            set { _request.Connection = value; }
        }

        /// <summary>
        /// Gets and sets the name of the connection group for this request
        /// </summary>
        public string ConnectionGroupName
        {
            get { return _request.ConnectionGroupName; }
            set { _request.ConnectionGroupName = value; }
        }

        /// <summary>
        /// Gets and sets the number of bytes of data to send to the Internet resource.
        /// </summary>
        public long ContentLength
        {
            get { return _request.ContentLength; }
            set { _request.ContentLength = value; }
        }

        /// <summary>
        /// Gets and sets the content type of the request.
        /// </summary>
        public string ContentType
        {
            get { return _request.ContentType; }
            set { _request.ContentType = value; }
        }

        /// <summary>
        /// Gets and sets the delegate that implements the callback method that 
        /// executes when an HTTP Continue response is returned from the Internet resource.
        /// </summary>
        public HttpContinueDelegate ContinueDelegate
        {
            get { return _request.ContinueDelegate; }
            set { _request.ContinueDelegate = value; }
        }

        /// <summary>
        /// Gets and sets cookies container associated with this request.
        /// </summary>
        public CookieContainer CookieContainer
        {
            get { return _request.CookieContainer; }
            set { _request.CookieContainer = value; }
        }

        /// <summary>
        /// Gets and sets the credentials used with the request.
        /// </summary>
        public ICredentials Credentials
        {
            get { return _request.Credentials; }
            set { _request.Credentials = value; }
        }

        /// <summary>
        /// Gets and sets the data header of the request.
        /// </summary>
        public DateTime Date
        {
            get { throw new NotImplementedException(); }
            set { throw new NotImplementedException(); }
        }

        /// <summary>
        /// Gets and sets the Expect HTTP header. 
        /// </summary>
        public string Expect
        {
            get { return _request.Expect; }
            set { _request.Expect = value; }
        }

        /// <summary>
        /// Gets  if a response has been received.
        /// </summary>
        public bool HaveResponse
        {
            get { return _request.HaveResponse; }
        }

        /// <summary>
        /// Gets and sets the headers of the request,
        /// </summary>
        public WebHeaderCollection Headers
        {
            get { return _request.Headers; }
            set { _request.Headers = value; }
        }

        /// <summary>
        /// Gets and sets the Host header.
        /// </summary>
        public string Host
        {
            get { throw new NotImplementedException(); }
            set { throw new NotImplementedException(); }
        }

        /// <summary>
        /// A DateTime that contains the contents of the If-Modified-Since HTTP header.
        /// </summary>
        public DateTime IfModifiedSince
        {
            get { return _request.IfModifiedSince; }
            set { _request.IfModifiedSince = value; }
        }

        /// <summary>
        /// Gets and sets the the impersonation level.
        /// </summary>
        public TokenImpersonationLevel ImpersonationLevel
        {
            get { return _request.ImpersonationLevel; }
            set { _request.ImpersonationLevel = value; }
        }

        /// <summary>
        /// Gets and sets if the request should have a Connection HTTP header.
        /// </summary>
        public bool KeepAlive
        {
            get { return _request.KeepAlive; }
            set { _request.KeepAlive = value; }
        }

        /// <summary>
        /// Gets and sets the max number of redirections.
        /// </summary>
        public int MaximumAutomaticRedirections
        {
            get { return _request.MaximumAutomaticRedirections; }
            set { _request.MaximumAutomaticRedirections = value; }
        }

        /// <summary>
        /// Gets and sets The length, in kilobytes (1024 bytes), of the response headers.
        /// </summary>
        public int MaximumResponseHeadersLength
        {
            get { return _request.MaximumResponseHeadersLength; }
            set { _request.MaximumResponseHeadersLength = value; }
        }

        /// <summary>
        /// The media type of the request. 
        /// </summary>
        public string MediaType
        {
            get { return _request.MediaType; }
            set { _request.MediaType = value; }
        }

        /// <summary>
        /// Gets and sets the request method.
        /// </summary>
        public string Method
        {
            get { return _request.Method; }
            set { _request.Method = value; }
        }

        /// <summary>
        /// Gets and sets if the request should be pipelined
        /// </summary>
        public bool Pipelined
        {
            get { return _request.Pipelined; }
            set { _request.Pipelined = value; }
        }

        /// <summary>
        /// Gets and sets if an HTTP Authorization header with requests after authentication has taken place;
        /// </summary>
        public bool PreAuthenticate
        {
            get { return _request.PreAuthenticate; }
            set { _request.PreAuthenticate = value; }
        }

        /// <summary>
        /// Gets and sets the protol version used.
        /// </summary>
        public Version ProtocolVersion
        {
            get { return _request.ProtocolVersion; }
            set { _request.ProtocolVersion = value; }
        }

        /// <summary>
        /// Gets and sets the proxy information.
        /// </summary>
        public IWebProxy Proxy
        {
            get { return _request.Proxy; }
            set { _request.Proxy = value; }
        }

        /// <summary>
        /// Gets and sets the number of milliseconds before the writing or reading times out..
        /// </summary>
        public int ReadWriteTimeout
        {
            get { return _request.ReadWriteTimeout; }
            set { _request.ReadWriteTimeout = value; }
        }

        /// <summary>
        /// Gets and sets the Referer HTTP header.
        /// </summary>
        public string Referer
        {
            get { return _request.Referer; }
            set { _request.Referer = value; }
        }

        /// <summary>
        /// Gets the resource uri used by the request.
        /// </summary>
        public Uri RequestUri
        {
            get { return _request.RequestUri; }
        }

        /// <summary>
        /// Ges and sets if the request should be sent in chuckes.
        /// </summary>
        public bool SendChunked
        {
            get { return _request.SendChunked; }
            set { _request.SendChunked = value; }
        }

        /// <summary>
        /// Gets the service point that represents the internet resource.
        /// </summary>
        public ServicePoint ServicePoint
        {
            get { return _request.ServicePoint; }
        }

        /// <summary>
        /// Gets and sets the number of milliseconds to wait before the request times out.
        /// </summary>
        public int Timeout
        {
            get { return _request.Timeout; }
            set { _request.Timeout = value; }
        }

        /// <summary>
        /// Gets and sets the Transfer-encoding HTTP header.
        /// </summary>
        public string TransferEncoding
        {
            get { return _request.TransferEncoding; }
            set { _request.TransferEncoding = value; }
        }

        /// <summary>
        /// Gets and sets if the authenticated connection is kept opened.
        /// </summary>
        public bool UnsafeAuthenticatedConnectionSharing
        {
            get { return _request.UnsafeAuthenticatedConnectionSharing; }
            set { _request.UnsafeAuthenticatedConnectionSharing = value; }
        }

        /// <summary>
        /// Gets and sets if the default credentials are used.
        /// </summary>
        public bool UseDefaultCredentials
        {
            get { return _request.UseDefaultCredentials; }
            set { _request.UseDefaultCredentials = value; }
        }

        /// <summary>
        /// Gets or sets the User-agent HTTP header;
        /// </summary>
        public string UserAgent
        {
            get { return _request.UserAgent; }
            set { _request.UserAgent = value; }
        }

        /// <summary>
        /// Cancels a request to a resource
        /// </summary>
        public void Abort()
        {
            _request.Abort();
        }

        /// <summary>
        /// Begins an asynchronous request for a Stream object to use to write data.
        /// </summary>
        /// <param name="callback">The delegate to execute.</param>
        /// <param name="state">Teh state object of the request. </param>
        /// <returns>An async result.</returns>
        public IAsyncResult BeginGetRequestStream(AsyncCallback callback, object state)
        {
            return _request.BeginGetRequestStream(callback, state);
        }

        /// <summary>
        /// Begins an asynchronous request to an Internet resource.
        /// </summary>
        /// <param name="callback">Teh deleaget.</param>
        /// <param name="state">Teh state object fo the response.</param>
        /// <returns>an asyn result.</returns>
        public IAsyncResult BeginGetResponse(AsyncCallback callback, object state)
        {
            return _request.BeginGetResponse(callback, state);
        }

        /// <summary>
        /// Ends an asynchronous request for a Stream object to use to write data.
        /// </summary>
        /// <param name="asyncResult">The result from the async request.</param>
        /// <returns>Teh stream from teh request.</returns>
        public Stream EndGetRequestStream(IAsyncResult asyncResult)
        {
            return EndGetRequestStream(asyncResult);
        }

        /// <summary>
        /// Ends an asynchronous request for a Stream object to use to write data and outputs
        ///  the TransportContext associated with the stream.
        /// </summary>
        /// <param name="asyncResult">The async result from the request stream.</param>
        /// <param name="context">The context of the request.</param>
        /// <returns>The request stream.</returns>
        public Stream EndGetRequestStream(IAsyncResult asyncResult, out TransportContext context)
        {
            return _request.EndGetRequestStream(asyncResult, out context);
        }

        /// <summary>
        /// Ends an asynchronous request to an Internet resource.
        /// </summary>
        /// <param name="asyncResult">The async result from the asyn response.</param>
        /// <returns>Teh response frm the internet resource.</returns>
        public WebResponse EndGetResponse(IAsyncResult asyncResult)
        {
            return _request.EndGetResponse(asyncResult);
        }

        /// <summary>
        /// Gets a Stream object to use to write request data.
        /// </summary>
        /// <returns>Teh stream to which write the request.</returns>
        public Stream GetRequestStream()
        {
            return _request.GetRequestStream();
        }

        /// <summary>
        /// Returns a response from an Internet resource.
        /// </summary>
        /// <returns>Teh response from the resource.</returns>
        public WebResponse GetResponse()
        {
            return _request.GetResponse();
        }

        #endregion
    }
}
