﻿/*
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System;
using System.IO;
using System.Net;
using System.Net.Cache;
using System.Net.Security;
using System.Security.Cryptography.X509Certificates;
using System.Security.Principal;

namespace Canonical.UbuntuOne.Common.Net
{
    /// <summary>
    /// Helper interface that wrapps the HttpWebRequest class from System.Net
    /// </summary>
    public interface IHttpWebRequest
    {
        /// <summary>
        /// Gets and sets value of the Accept HTTP header. The default value is null.
        /// </summary>
        string Accept { get; set; }

        /// <summary>
        /// Gets and sets the uri that responds to the request.
        /// </summary>
        Uri Address { get; }

        /// <summary>
        /// Gets and sets if the request should automatically follow redirection responses
        /// </summary>
        bool AllowAutoRedirect { get; set; }

        /// <summary>
        /// Enables buffering of the data sent to the Internet resource
        /// </summary>
        bool AllowWriteStreamBuffering { get; set; }

        /// <summary>
        /// Gets and sets the auhentication lavel used.
        /// </summary>
        AuthenticationLevel AuthenticationLevel { get; set; }

        /// <summary>
        /// Gets and sets the type of decompression that is used. 
        /// </summary>
        DecompressionMethods AutomaticDecompression { get; set; }

        /// <summary>
        /// Gets or sets the cache policy used by the request.
        /// </summary>
        RequestCachePolicy CachePolicy { get; set; }

        /// <summary>
        /// Gets and sets the certificates used by the request.
        /// </summary>
        X509CertificateCollection ClientCertificates { get; set; }

        /// <summary>
        /// Gets and sets the Connection HTTP header
        /// </summary>
        string Connection { get; set; }

        /// <summary>
        /// Gets and sets the name of the connection group for this request
        /// </summary>
        string ConnectionGroupName { get; set; }

        /// <summary>
        /// Gets and sets the number of bytes of data to send to the Internet resource.
        /// </summary>
        long ContentLength { get; set; }

        /// <summary>
        /// Gets and sets the content type of the request.
        /// </summary>
        string ContentType { get; set; }

        /// <summary>
        /// Gets and sets the delegate that implements the callback method that 
        /// executes when an HTTP Continue response is returned from the Internet resource.
        /// </summary>
        HttpContinueDelegate ContinueDelegate { get; set; }

        /// <summary>
        /// Gets and sets cookies container associated with this request.
        /// </summary>
        CookieContainer CookieContainer { get; set; }

        /// <summary>
        /// Gets and sets the credentials used with the request.
        /// </summary>
        ICredentials Credentials { get; set; }

        /// <summary>
        /// Gets and sets the data header of the request.
        /// </summary>
        DateTime Date { get; set; }

        /// <summary>
        /// Gets and sets the Expect HTTP header. 
        /// </summary>
        string Expect { get; set; }

        /// <summary>
        /// Gets  if a response has been received.
        /// </summary>
        bool HaveResponse { get; }

        /// <summary>
        /// Gets and sets the headers of the request,
        /// </summary>
        WebHeaderCollection Headers { get; set; }

        /// <summary>
        /// Gets and sets the Host header.
        /// </summary>
        string Host { get; set; }

        /// <summary>
        /// A DateTime that contains the contents of the If-Modified-Since HTTP header.
        /// </summary>
        DateTime IfModifiedSince { get; set; }

        /// <summary>
        /// Gets and sets the the impersonation level.
        /// </summary>
        TokenImpersonationLevel ImpersonationLevel { get; set; }

        /// <summary>
        /// Gets and sets if the request should have a Connection HTTP header.
        /// </summary>
        bool KeepAlive { get; set; }

        /// <summary>
        /// Gets and sets the max number of redirections.
        /// </summary>
        int MaximumAutomaticRedirections { get; set; }

        /// <summary>
        /// Gets and sets The length, in kilobytes (1024 bytes), of the response headers.
        /// </summary>
        int MaximumResponseHeadersLength { get; set; }

        /// <summary>
        /// The media type of the request. 
        /// </summary>
        string MediaType { get; set; }

        /// <summary>
        /// Gets and sets the request method.
        /// </summary>
        string Method { get; set; }

        /// <summary>
        /// Gets and sets if the request should be pipelined
        /// </summary>
        bool Pipelined { get; set; }

        /// <summary>
        /// Gets and sets if an HTTP Authorization header with requests after authentication has taken place;
        /// </summary>
        bool PreAuthenticate { get; set; }

        /// <summary>
        /// Gets and sets the protol version used.
        /// </summary>
        Version ProtocolVersion { get; set; }

        /// <summary>
        /// Gets and sets the proxy information.
        /// </summary>
        IWebProxy Proxy { get; set; }

        /// <summary>
        /// Gets and sets the number of milliseconds before the writing or reading times out..
        /// </summary>
        int ReadWriteTimeout { get; set; }

        /// <summary>
        /// Gets and sets the Referer HTTP header.
        /// </summary>
        string Referer { get; set; }

        /// <summary>
        /// Gets the resource uri used by the request.
        /// </summary>
        Uri RequestUri { get; }

        /// <summary>
        /// Ges and sets if the request should be sent in chuckes.
        /// </summary>
        bool SendChunked { get; set; }

        /// <summary>
        /// Gets the service point that represents the internet resource.
        /// </summary>
        ServicePoint ServicePoint { get; }

        /// <summary>
        /// Gets and sets the number of milliseconds to wait before the request times out.
        /// </summary>
        int Timeout { get; set; }

        /// <summary>
        /// Gets and sets the Transfer-encoding HTTP header.
        /// </summary>
        string TransferEncoding { get; set; }

        /// <summary>
        /// Gets and sets if the authenticated connection is kept opened.
        /// </summary>
        bool UnsafeAuthenticatedConnectionSharing { get; set; }

        /// <summary>
        /// Gets and sets if the default credentials are used.
        /// </summary>
        bool UseDefaultCredentials { get; set; }

        /// <summary>
        /// Gets or sets the User-agent HTTP header;
        /// </summary>
        string UserAgent { get; set; }

        /// <summary>
        /// Cancels a request to a resource
        /// </summary>
        void Abort();

        /// <summary>
        /// Begins an asynchronous request for a Stream object to use to write data.
        /// </summary>
        /// <param name="callback">The delegate to execute.</param>
        /// <param name="state">Teh state object of the request. </param>
        /// <returns>An async result.</returns>
        IAsyncResult BeginGetRequestStream(AsyncCallback callback,Object state);

        /// <summary>
        /// Begins an asynchronous request to an Internet resource.
        /// </summary>
        /// <param name="callback">Teh deleaget.</param>
        /// <param name="state">Teh state object fo the response.</param>
        /// <returns>an asyn result.</returns>
        IAsyncResult BeginGetResponse(AsyncCallback callback,Object state);

        /// <summary>
        /// Ends an asynchronous request for a Stream object to use to write data.
        /// </summary>
        /// <param name="asyncResult">The result from the async request.</param>
        /// <returns>Teh stream from teh request.</returns>
        Stream EndGetRequestStream(IAsyncResult asyncResult);

        /// <summary>
        /// Ends an asynchronous request for a Stream object to use to write data and outputs
        ///  the TransportContext associated with the stream.
        /// </summary>
        /// <param name="asyncResult">The async result from the request stream.</param>
        /// <param name="context">The context of the request.</param>
        /// <returns>The request stream.</returns>
        Stream EndGetRequestStream(IAsyncResult asyncResult,out TransportContext context);

        /// <summary>
        /// Ends an asynchronous request to an Internet resource.
        /// </summary>
        /// <param name="asyncResult">The async result from the asyn response.</param>
        /// <returns>Teh response frm the internet resource.</returns>
        WebResponse EndGetResponse(IAsyncResult asyncResult);

        /// <summary>
        /// Gets a Stream object to use to write request data.
        /// </summary>
        /// <returns>Teh stream to which write the request.</returns>
        Stream GetRequestStream();

        /// <summary>
        /// Returns a response from an Internet resource.
        /// </summary>
        /// <returns>Teh response from the resource.</returns>
        WebResponse GetResponse();
    }
}
