﻿/*
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System.Threading;
using Microsoft.ApplicationBlocks.Updater;

namespace Canonical.UbuntuOne.Common.Update
{
    /// <summary>
    /// Presenter that manages Downloading views.
    /// </summary>
    public class DownloadingPresenter : IDownloadingPresenter
    {
        #region Variables

        private IDownloadingView _view;
        /// <summary>
        /// The maximum value to be used in the view related to the percentage of 
        /// update that has been downloaded.
        /// </summary>
        public const double MaximumValue = 100;

        /// <summary>
        /// The minimum value to be used in the view related to the percentage of
        /// update that has been downloaded.
        /// </summary>
        public const double MinimumValue = 0;

        #endregion

        /// <summary>
        /// Gets and sets the view to be used to interact with the user.
        /// </summary>
        public IDownloadingView View
        {
            get { return _view; }
            set
            {
                if (value != null)
                {
                    var progress = (_view != null) ? _view.Progress : 0;
                    _view = value;
                    _view.MaximumValue = MaximumValue;
                    _view.MinimumValue = MinimumValue;
                    _view.Progress = progress;
                }
                else
                {
                    _view = null;
                }
            }
        }

        /// <summary>
        /// Handler to be called when there has been an updated related to the progess of the download
        /// of the update.
        /// </summary>
        /// <param name="sender">The object that sent the event.</param>
        /// <param name="args">The arguments that provide information regarding the changes in the download.</param>
        public void DownloadProgressHandler(object sender, DownloadTaskProgressEventArgs args)
        {
            if (!View.IsShown)
            {
                // dispatch to the correct thread to show the view
                View.ViewDispatcher.Dispatch(() => View.Show());
            }
            // dispatch to the correct thread to set the data
            View.ViewDispatcher.Dispatch(
                () =>
                    {
                        View.DownloadingMessage = UpdateResources.DownloadInProgress;
                        View.Progress = (args.BytesTransferred * MaximumValue) / args.BytesTotal;
                    });
        }

        /// <summary>
        /// Handler to be called when there has been an error when downloading an update.
        /// </summary>
        /// <param name="sender">The object that sent the event.</param>
        /// <param name="args">The arguments that provide information regarding the error.</param>
        public void DownloadErrorHandler(object sender, DownloadTaskErrorEventArgs args)
        {
            View.ViewDispatcher.Dispatch(
                () =>
                    {
                        View.DownloadingMessage = UpdateResources.DownloadError;
                        View.Progress = MinimumValue;
                        View.CloseButtonIsEnabled = true;
                    });
        }

        /// <summary>
        /// Handler to be called when the update has been completed.
        /// </summary>
        /// <param name="sender">The object that sent the event.</param>
        /// <param name="e">The arguments that provide more information about the comleted task.</param>
        public void DownloadUpdateCompleted(object sender, TaskEventArgs e)
        {
            View.ViewDispatcher.Dispatch(
                () =>
                    {
                        View.DownloadingMessage = UpdateResources.DownloadCompleted;
                        View.Progress = MaximumValue;
                    });
            // let the user know
            Thread.Sleep(100);
            View.ViewDispatcher.Dispatch(() => View.Close());
        }
    }
}
