﻿/*
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System;
using System.Windows;
using System.Windows.Threading;
using DotUpdater;
using log4net;
using Microsoft.ApplicationBlocks.Updater;

namespace Canonical.UbuntuOne.Common.Update
{
    /// <summary>
    /// Implementation of the IUpdater interface that will be wrapping the DotUpdater classes
    /// from http://dotupdater.codeplex.com/
    /// </summary>
    public class Updater : IUpdater
    {

        #region Variables

        private IDotUpdaterWrapper _updater;
        private readonly object _updaterLock = new object();
        private UpdaterTask _task;
        private ILog _logger;
        private readonly object _loggerLock = new object();
        private bool _updateChecked;
        private readonly object _updateLock = new object();
        private bool _downloadingUpdate;

        #endregion

        #region Delegates

        delegate void UpdaterDelegate(UpdaterTask task);

        #endregion

        #region DI properties

        /// <summary>
        /// Gets and sets the url of the feed that is used to determine if there are updates available.
        /// </summary>
        public string RSSFeed { get; set; }

        /// <summary>
        /// Gets and sets the message box that is used to communicate messages to the user,
        /// </summary>
        public IMessageBox MessageBox { get; set; }

        /// <summary>
        /// Gets and sets the downloading presenter that manages the view used to tell the user
        /// that the updates are being downloaded.
        /// </summary>
        public IDownloadingPresenter DownloadingPresenter { get; set; }

        /// <summary>
        /// Gets and sets the updating presenter that manages the view used to tell the user 
        /// that the updates are taking place.
        /// </summary>
        public IUpdatingPresenter UpdatingPresenter { get; set; }

        #endregion

        #region Helper properties

        /// <summary>
        /// Gets and sets the DotUpdater used by the class to perform the different 
        /// operaqtions, this allows to later use mocks when testing.
        /// </summary>
        internal IDotUpdaterWrapper DotUpdater
        {
            get
            {
                if(_updater == null)
                {
                    lock (_updaterLock)
                    {
                        if(String.IsNullOrEmpty(RSSFeed))
                        {
                            Logger.Error(UpdateResources.NoRSSFeedPresent);
                            throw new SelfUpdateException(UpdateResources.NoRSSFeedPresent);
                        }
                        _updater = new DotUpdaterWrapper(new DotLogger(), RSSFeed);
                    }
                }
                return _updater;
            }
            set
            {
                lock (_updaterLock)
                {
                    _updater = value;
                }
            }
        }

        /// <summary>
        /// Gets and sets the logger that is used by the class.
        /// </summary>
        public ILog Logger
        {
            get
            {
                return _logger ?? (_logger = LogManager.GetLogger(typeof (Updater)));
            }
            set
            {
                lock (_loggerLock)
                {
                    _logger = value;
                }
            }
        }
        #endregion

        #region Helper methods

        /// <summary>
        /// Helper that will dispatch the update operation to a different thread to be performed.
        /// </summary>
        /// <param name="sender">The object that sent the event args.</param>
        /// <param name="e">The arguments of the event.</param>
        private void DownloadUpdateCompleted(object sender, TaskEventArgs e)
        {
            Logger.Info(UpdateResources.DownloadCompleted);
            // call the method in an other thread that will perform the update
            UpdateDownloadedMainThread(e.Task);
        }

        /// <summary>
        /// Helper that will take care of cleaning up and logging after an expcetion has occurred when
        /// the download failed.
        /// </summary>
        /// <param name="sender">The object that sent the event.</param>
        /// <param name="args">The argumens of the event.</param>
        private void DownloadErrorHandler(object sender, DownloadTaskErrorEventArgs args)
        {
            Logger.Error(args.Exception.Message);
            throw new SelfUpdateException(UpdateResources.DownloadError, args.Exception);
        }

        /// <summary>
        /// Helper method that will perform the actual update of the application, to do so the updater will
        /// close and update the application adter asking the user throw the view.
        /// </summary>
        /// <param name="task">The updater tasks that was being performed.</param>
        private void UpdateDownloadedMainThread(UpdaterTask task)
        {
            if (MessageBox.ShowOkCancel(UpdateResources.PerformUpdateQuestion, MessageBoxImage.Question) == MessageBoxResult.Cancel ) 
                return;

            try
            {
                UpdatingPresenter.UpdatingStarted();
                // perform the update
                DotUpdater.Install(_task);
                // clean the resoruces
                _task = null;
                _updateChecked = false;
                _downloadingUpdate = false;
                UpdatingPresenter.UpdatingFinished();
                MessageBox.ShowInformation(UpdateResources.UpdatesSucceded);
            }
            catch(UpdaterException)
            {
                MessageBox.ShowError(UpdateResources.UpdatesError);
            }

        }

        #endregion

        /// <summary>
        /// Returns if the are updates present for the application.
        /// </summary>
        /// <returns>A flag indicating if updates are available.</returns>
        public bool UpdatesArePresent()
        {
            try
            {
                // we lock since this is done in a multithreaded enviroment and 
                // we want to make sure that we do not check for the update twice at the same time
                if (_task == null)
                {
                    lock (_updateLock)
                    {
                        _task = DotUpdater.CheckForUpdates();
                        _updateChecked = true;
                    }
                }
                return _task != null;
            }
            catch(UpdaterException ex)
            {
                throw new SelfUpdateException("Could not check for updates.", ex);
            }
        }

        /// <summary>
        /// This method will perform the update of the application using a msi installer that
        /// is fecthed from the rss feed.
        /// </summary>
        public void PerformUpdate()
        {
            lock (_updateLock)
            {
                // ask the user if he wants to perform the update
                if (MessageBox.ShowYesNo(UpdateResources.DownloadUpdatesQuestion, MessageBoxImage.Question) == MessageBoxResult.No) return;
                if (!_updateChecked)
                {
                    Logger.Error(UpdateResources.CheckNotDone);
                    throw new SelfUpdateException(UpdateResources.CheckNotDone);
                }
                if (_task == null)
                {
                    Logger.Error(UpdateResources.NoUpdatesAvailable);
                    throw new SelfUpdateException(UpdateResources.NoUpdatesAvailable);
                }
                if (!_task.Downloaded && !_downloadingUpdate)
                {
                    // we need to download the update
                    try
                    {
                        _downloadingUpdate = true;
                        // provide the handler for the progress of the download as well as the 
                        // error and finish
                        DotUpdater.OnDownloadProgress += DownloadingPresenter.DownloadProgressHandler;
                        // we provide two handlers one for the view to let the user know we completed
                        // the download and a second one for the updater that will do the actual work
                        DotUpdater.OnDownloadCompleted += DownloadingPresenter.DownloadUpdateCompleted;
                        DotUpdater.OnDownloadCompleted += DownloadUpdateCompleted;
                        // register to handlers, one for the view that will show the error and an other
                        // one for the updater that will handle the error.
                        DotUpdater.OnDownloadError += DownloadingPresenter.DownloadErrorHandler;
                        DotUpdater.OnDownloadError += DownloadErrorHandler;
                        DotUpdater.BeginDownload(_task);
                    }
                    catch (UpdaterException e)
                    {
                        Logger.Error(UpdateResources.CannotDownload);
                        throw new SelfUpdateException(UpdateResources.CannotDownload, e);
                    }
                }
                if (_task.Downloaded)
                {
                    _downloadingUpdate = false;
                    // just call the handler directly
                    DownloadUpdateCompleted(this, new TaskEventArgs(_task));
                }
            }
        }

        /// <summary>
        /// Defines a method to release allocated resources.
        /// </summary>
        /// <remarks>
        /// The primary use of this interface is to release unmanaged resources. The garbage collector 
        /// automatically releases the memory allocated to a managed object when that object is no longer 
        /// used. However, it is not possible to predict when garbage collection will occur. Furthermore, 
        /// the garbage collector has no knowledge of unmanaged resources such as window handles, 
        /// or open files and streams.
        ///
        /// Use the Dispose method of this interface to explicitly release unmanaged resources in 
        /// conjunction with the garbage collector. The consumer of an object can call this method 
        /// when the object is no longer needed.
        /// </remarks>
        public void Dispose()
        {
            if (_updater == null) return;
            _updater.Dispose();
            _updater = null;
        }
    }
}
