﻿/*
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System.Collections.Generic;
using System.Windows;

namespace Canonical.UbuntuOne.Common.Utils
{
    /// <summary>
    /// Wrapper that allows to add an abstraction when working with the System.Window.Application class.
    /// </summary>
    public class ApplicationWrapper : IApplication
    {
        private readonly Application _app;

        /// <summary>
        /// Initializes a new instance of the ApplicationWrapper class with a new application.
        /// </summary>
        /// <remarks>
        /// At any point of time just a single ApplicationWrapper instance can be present in an application.
        /// </remarks>
        public ApplicationWrapper()
        {
            _app = new Application();
            StartupTasks = new List<IStartupTask>();
        }

        #region Implementation of IApplication

        /// <summary>
        /// Gets and sets the tasks that will be executed in sequencial 
        /// order during the start up of the application.
        /// </summary>
        public IList<IStartupTask> StartupTasks { get; set; }

        /// <summary>
        /// Allows to start the application using the passed window.
        /// </summary>
        /// <param name="startWindow">The main window of the application.</param>
        public void Run(Window startWindow)
        {
            // execute the actions that need to be performed before the app is started.
            if (StartupTasks != null)
            {
                foreach (var startupTask in StartupTasks)
                {
                    startupTask.Exeucte();
                }
            }

            _app.Run(startWindow);
        }

        /// <summary>
        /// Gets and sets the shutdown mode to be used in the application.
        /// </summary>
        public ShutdownMode ShutdownMode
        {
            get { return _app.ShutdownMode; }
            set { _app.ShutdownMode = value; }
        }

        /// <summary>
        /// Allows to close the application.
        /// </summary>
        public void Exit()
        {
            _app.Shutdown();
        }

        #endregion
    }
}
