﻿/*
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System;
using System.Collections.Generic;
using System.IO;
using Canonical.UbuntuOne.Common.Win32;
using Canonical.UbuntuOne.ProcessDispatcher;
using log4net;
using Microsoft.Win32;

namespace Canonical.UbuntuOne.Common.Utils
{
    /// <summary>
    /// Implementation of an IApplicationLocator that does know how to locate the U1 applictions.
    /// </summary>
    public class UbuntuOneApplicationLocator : IApplicationLocator
    {
        #region Variables

        private readonly IRegistryKey _localMachineKey;
        private string _installationPath;
        private readonly object _installationPathLock = new object();
        private ILog _logger;
        private readonly object _loggerLock = new object();

        #endregion

        #region Constructors

        public UbuntuOneApplicationLocator()
            : this(new RegistryKeyWrapper(Registry.LocalMachine))
        {

        }
        public UbuntuOneApplicationLocator(IRegistryKey registryKey)
        {
            _localMachineKey = registryKey;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets and sets the logger used by the class.
        /// </summary>
        internal ILog Logger
        {
            get
            {
                if (_logger == null)
                {
                    lock (_loggerLock)
                    {
                        _logger = LogManager.GetLogger(typeof(UbuntuOneApplicationLocator));
                    }
                }
                return _logger;
            }
            set { _logger = value; }
        }

        #endregion

        #region Helpers

        private string InstallationPath
        {
            get
            {
                if (_installationPath == null)
                {
                    lock (_installationPathLock)
                    {
                        try
                        {
                            using (var softwareEntry = _localMachineKey.OpenSubKey("SOFTWARE"))
                            using (var canonicalEntry = softwareEntry.OpenSubKey("Canonical"))
                            using (var ubuntuOneEntry = canonicalEntry.OpenSubKey("UbuntuOne"))
                            {
                                _installationPath = (string)ubuntuOneEntry.GetValue("InstallationPath");
                            }
                        }
                        catch (NullReferenceException e)
                        {
                            // thrown when we did not find the key, that is, not correct installation
                            Logger.Error("Registry key HKLM/Software/Canonical/UbuntuOne is not present!!!");
                            throw new ApplicationLocatorException("Registry key HKLM/Software/Canonical/UbuntuOne is not present!!!", e);
                        }
                    }
                }
                return _installationPath;
            }
        }
        #endregion

        #region Implementation of IApplicationLocator

        /// <summary>
        /// Returns a collecion with the names of the known application by the object.
        /// </summary>
        /// <returns>A string with all the different applications that the object knows how to locate.</returns>
        public ICollection<string> GetKnownApplications()
        {
            return new[] { Constants.U1SyncAppName, Constants.U1DaemonAppName, Constants.U1ClientAppName };
        }

        /// <summary>
        /// Returns the full path in the system of the application that mathces the given name.
        /// </summary>
        /// <param name="appName">The name of the application we want to locate.</param>
        /// <returns>The full path of the application with the given name.</returns>
        public string GetApplicationPath(string appName)
        {
            if (string.Compare(appName, Constants.U1SyncAppName, true) == 0)
            {
                var u1sync = Path.Combine(InstallationPath, "Client");
                u1sync = Path.Combine(u1sync, "U1Sync");
                return u1sync;
            }
            if (string.Compare(appName, Constants.U1DaemonAppName, true) == 0)
            {
                var u1daemon = Path.Combine(InstallationPath, "Daemon");
                return u1daemon;
            }
            if(string.Compare(appName, Constants.U1ClientAppName, true) == 0)
            {
                var u1client = Path.Combine(InstallationPath, "Client");
                return u1client;
            }
            throw new ApplicationLocatorException(string.Format("Application {0} is not known.", appName));
        }

        #endregion
    }
}
