﻿/*
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */

using System;
using Microsoft.Win32;

namespace Canonical.UbuntuOne.Common.Win32
{
    /// <summary>
    /// Interface that represents a class that allows to perform the access to 
    /// the registry keys of the machine.
    /// </summary>
    public interface IRegistryKey : IDisposable
    {
        #region Properties

        /// <summary>
        /// Retrieves the name of the key.
        /// </summary>
        string Name { get; }

        /// <summary>
        /// Retrieves the count of subkeys of the current key.
        /// </summary>
        int SubKeyCount { get; }

        /// <summary>
        /// Retrieves the count of values in the key.
        /// </summary>
        int ValueCount { get; }

        #endregion

        #region Methods

        /// <summary>
        /// Closes the key and flushes it to disk if its contents have been modified.
        /// </summary>
        void Close();

        /// <summary>
        /// Creates a new subkey or opens an existing subkey for write access. 
        /// The string subkey is not case-sensitive.
        /// </summary>
        /// <param name="name">The name or path of the subkey to create or open. </param>
        /// <returns>A sub key to work with.</returns>
        IRegistryKey CreateSubKey(string name);

        /// <summary>
        /// Creates a new subkey or opens an existing subkey for write access, using the specified 
        /// permission check option. The string subkey is not case-sensitive.
        /// </summary>
        /// <param name="subkey">The name or path of the subkey to create or open.</param>
        /// <param name="permissionCheck">One of the RegistryKeyPermissionCheck values that 
        /// specifies whether the key is opened for read or read/write access.</param>
        /// <returns>A subkey to work with.</returns>
        IRegistryKey CreateSubKey(string subkey, RegistryKeyPermissionCheck permissionCheck);
        
        /// <summary>
        /// Deletes the specified subkey. The string subkey is not case-sensitive.
        /// </summary>
        /// <param name="subkey">The name of the subkey to delete. </param>
        void DeleteSubKey(string subkey);

        /// <summary>
        /// Deletes the specified value from this key.
        /// </summary>
        /// <param name="name">The name of the value to delete.</param>
        void DeleteValue(string name);

        /// <summary>
        /// Writes all the attributes of the specified open registry 
        /// key into the registry.
        /// </summary>
        void Flush();

        /// <summary>
        /// Retrieves an array of strings that contains all the subkey names.
        /// </summary>
        /// <returns>An array of strings that contains the names of the 
        /// subkeys for the current key.</returns>
        string[] GetSubKeyNames();

        /// <summary>
        /// Retrieves the value associated with the specified name. 
        /// Returns null if the name/value pair does not exist in the registry.
        /// </summary>
        /// <param name="name">The name of the value to retrieve. </param>
        /// <returns>The value associated with name, or null if name is not found.</returns>
        object GetValue(string name);

        /// <summary>
        /// Retrieves the value associated with the specified name. 
        /// If the name is not found, returns the default value that you provide.
        /// </summary>
        /// <param name="name">The name of the value to retrieve.</param>
        /// <param name="defaultValue">The value to return if name does not exist.</param>
        /// <returns>The value associated with name, with any embedded 
        /// environment variables left unexpanded, or defaultValue if name is not found.</returns>
        object GetValue(string name, object defaultValue);

        /// <summary>
        /// Retrieves the registry data type of the value associated with the specified name.
        /// </summary>
        /// <param name="name">The name of the value whose registry data type is to be retrieved. </param>
        /// <returns>A RegistryValueKind value representing the registry 
        /// data type of the value associated with name.</returns>
        RegistryValueKind GetValueKind(string name);

        /// <summary>
        /// Retrieves an array of strings that contains all the 
        /// value names associated with this key.
        /// </summary>
        /// <returns>An array of strings that contains 
        /// the value names for the current key.</returns>
        string[] GetValueNames();

        /// <summary>
        /// Retrieves a subkey as read-only.
        /// </summary>
        /// <param name="name">The name or path of the subkey to open read-only. </param>
        /// <returns>The subkey requested, or null if the operation failed.</returns>
        IRegistryKey OpenSubKey(string name);

        /// <summary>
        /// Retrieves the specified subkey for read or read/write access.
        /// </summary>
        /// <param name="name">The name or path of the subkey to create or open.</param>
        /// <param name="permissionCheck">One of the RegistryKeyPermissionCheck values 
        /// that specifies whether the key is opened for read or read/write access.</param>
        /// <returns></returns>
        IRegistryKey OpenSubKey(string name, RegistryKeyPermissionCheck permissionCheck);

        /// <summary>
        /// Sets the specified name/value pair.
        /// </summary>
        /// <param name="name">The name of the value to store. </param>
        /// <param name="value">The data to be stored.</param>
        void SetValue(string name, Object value);

        /// <summary>
        /// Sets the value of a name/value pair in the registry key, using the specified registry data type.
        /// </summary>
        /// <param name="name">The name of the value to be stored. </param>
        /// <param name="value">The data to be stored. </param>
        /// <param name="valueKind">The registry data type to use when storing the data</param>
        void SetValue(string name, Object value, RegistryValueKind valueKind);

        #endregion



    }
}
