﻿/*
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */

using System;
using Canonical.UbuntuOne.Common.Aop;
using log4net;
using Microsoft.Win32;

namespace Canonical.UbuntuOne.Common.Win32
{
    /// <summary>
    /// Simple wrapper around the registry class that allows the class to be mocked for testing 
    /// purposes
    /// </summary>
    public class RegistryKeyWrapper : IRegistryKey
    {
        #region Variables

        private readonly RegistryKey _key;
        private ILog _logger;
        private readonly object _loggerLock = new object();

        #endregion

        #region DI properties

        /// <summary>
        /// Gets and sets the logger that will be used to log the operation of the
        /// class.
        /// </summary>
        public ILog Logger
        {
            get
            {
                if (_logger == null)
                {
                    lock (_loggerLock)
                    {
                        _logger = LogManager.GetLogger(typeof(RegistryKeyWrapper));
                    }
                }
                return _logger;
            }
            set
            {
                _logger = value;
            }
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Creates a new registry key that contains information about the default user configuration.
        /// </summary>
        public RegistryKeyWrapper()
            : this(Registry.CurrentUser)
        {

        }

        /// <summary>
        /// Creates a new registry key wrapper with the given key.
        /// </summary>
        /// <param name="key">The key to be wrapped.</param>
        public RegistryKeyWrapper(RegistryKey key)
        {
            _key = key;
        }

        #endregion

        #region Implementation of IDisposable

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, 
        /// or resetting unmanaged resources.
        /// </summary>
        /// <filterpriority>2</filterpriority>
        public void Dispose()
        {
            var disposable = (IDisposable)_key;
            disposable.Dispose();
        }

        #endregion

        #region Implementation of IRegistryKey

        /// <summary>
        /// Retrieves the name of the key.
        /// </summary>
        public string Name
        {
            get { return _key.Name; }
        }

        /// <summary>
        /// Retrieves the count of subkeys of the current key.
        /// </summary>
        public int SubKeyCount
        {
            get { return _key.SubKeyCount; }
        }

        /// <summary>
        /// Retrieves the count of values in the key.
        /// </summary>
        public int ValueCount
        {
            get { return _key.ValueCount; }
        }

        /// <summary>
        /// Closes the key and flushes it to disk if its contents have been modified.
        /// </summary>
        public void Close()
        {
            _key.Close();
        }

        /// <summary>
        /// Creates a new subkey or opens an existing subkey for write access. 
        /// The string subkey is not case-sensitive.
        /// </summary>
        /// <param name="name">The name or path of the subkey to create or open. </param>
        /// <returns>A sub key to work with.</returns>
        [DebugLogged]
        public IRegistryKey CreateSubKey(string name)
        {
            return new RegistryKeyWrapper(_key.CreateSubKey(name));
        }

        /// <summary>
        /// Creates a new subkey or opens an existing subkey for write access, using the specified 
        /// permission check option. The string subkey is not case-sensitive.
        /// </summary>
        /// <param name="subkey">The name or path of the subkey to create or open.</param>
        /// <param name="permissionCheck">One of the RegistryKeyPermissionCheck values that 
        /// specifies whether the key is opened for read or read/write access.</param>
        /// <returns>A subkey to work with.</returns>
        [DebugLogged]
        public IRegistryKey CreateSubKey(string subkey, RegistryKeyPermissionCheck permissionCheck)
        {
            return new RegistryKeyWrapper(_key.CreateSubKey(subkey, permissionCheck));
        }

        /// <summary>
        /// Deletes the specified subkey. The string subkey is not case-sensitive.
        /// </summary>
        /// <param name="subkey">The name of the subkey to delete. </param>
        [DebugLogged]
        public void DeleteSubKey(string subkey)
        {
            _key.DeleteSubKey(subkey);
        }

        /// <summary>
        /// Deletes the specified value from this key.
        /// </summary>
        /// <param name="name">The name of the value to delete.</param>
        [DebugLogged]
        public void DeleteValue(string name)
        {
            _key.DeleteValue(name);
        }

        /// <summary>
        /// Writes all the attributes of the specified open registry 
        /// key into the registry.
        /// </summary>
        [DebugLogged]
        public void Flush()
        {
            _key.Flush();
        }

        /// <summary>
        /// Retrieves an array of strings that contains all the subkey names.
        /// </summary>
        /// <returns>An array of strings that contains the names of the 
        /// subkeys for the current key.</returns>
        [DebugLogged]
        public string[] GetSubKeyNames()
        {
            return _key.GetSubKeyNames();
        }

        /// <summary>
        /// Retrieves the value associated with the specified name. 
        /// Returns null if the name/value pair does not exist in the registry.
        /// </summary>
        /// <param name="name">The name of the value to retrieve. </param>
        /// <returns>The value associated with name, or null if name is not found.</returns>
        public object GetValue(string name)
        {
            // use norml logging because otherwhise it will expose user data
            Logger.DebugFormat("> GetValue({0})", name);
            return _key.GetValue(name);
        }

        /// <summary>
        /// Retrieves the value associated with the specified name. 
        /// If the name is not found, returns the default value that you provide.
        /// </summary>
        /// <param name="name">The name of the value to retrieve.</param>
        /// <param name="defaultValue">The value to return if name does not exist.</param>
        /// <returns>The value associated with name, with any embedded 
        /// environment variables left unexpanded, or defaultValue if name is not found.</returns>
        public object GetValue(string name, object defaultValue)
        {
            Logger.DebugFormat("> GetValue({0}, {1})", name, defaultValue);
            return _key.GetValue(name, defaultValue);
        }

        /// <summary>
        /// Retrieves the registry data type of the value associated with the specified name.
        /// </summary>
        /// <param name="name">The name of the value whose registry data type is to be retrieved. </param>
        /// <returns>A RegistryValueKind value representing the registry 
        /// data type of the value associated with name.</returns>
        [DebugLogged]
        public RegistryValueKind GetValueKind(string name)
        {
            return _key.GetValueKind(name);
        }

        /// <summary>
        /// Retrieves an array of strings that contains all the 
        /// value names associated with this key.
        /// </summary>
        /// <returns>An array of strings that contains 
        /// the value names for the current key.</returns>
        [DebugLogged]
        public string[] GetValueNames()
        {
            return _key.GetValueNames();
        }

        /// <summary>
        /// Retrieves a subkey as read-only.
        /// </summary>
        /// <param name="name">The name or path of the subkey to open read-only. </param>
        /// <returns>The subkey requested, or null if the operation failed.</returns>
        [DebugLogged]
        public IRegistryKey OpenSubKey(string name)
        {
            return new RegistryKeyWrapper(_key.OpenSubKey(name));
        }

        /// <summary>
        /// Retrieves the specified subkey for read or read/write access.
        /// </summary>
        /// <param name="name">The name or path of the subkey to create or open.</param>
        /// <param name="permissionCheck">One of the RegistryKeyPermissionCheck values 
        /// that specifies whether the key is opened for read or read/write access.</param>
        /// <returns></returns>
        [DebugLogged]
        public IRegistryKey OpenSubKey(string name, RegistryKeyPermissionCheck permissionCheck)
        {
            return new RegistryKeyWrapper(_key.OpenSubKey(name, permissionCheck));
        }

        /// <summary>
        /// Sets the specified name/value pair.
        /// </summary>
        /// <param name="name">The name of the value to store. </param>
        /// <param name="value">The data to be stored.</param>
        public void SetValue(string name, object value)
        {
            Logger.DebugFormat("> SetValue({0}, \"******\")", name);
            _key.SetValue(name, value);
        }

        /// <summary>
        /// Sets the value of a name/value pair in the registry key, using the specified registry data type.
        /// </summary>
        /// <param name="name">The name of the value to be stored. </param>
        /// <param name="value">The data to be stored. </param>
        /// <param name="valueKind">The registry data type to use when storing the data</param>
        public void SetValue(string name, object value, RegistryValueKind valueKind)
        {
            Logger.DebugFormat("> SetValue({0}, \"******\", {1})", name, valueKind);
            _key.SetValue(name, value, valueKind);
        }

        #endregion
    }
}
