﻿/*
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System;
using System.IO;
using System.Text;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace Canonical.UbuntuOne.ProcessDispatcher
{
    /// <summary>
    /// Pipe streamer that uses json to talk with the python code.
    /// </summary>
    internal class JsonPipeStreamer : IPipeStreamer
    {
        #region Implementation of IPipeStreamer

        /// <summary>
        /// Gets and sets the encoding used by the streamer.
        /// </summary>
        public Encoding Encoding
        {
            get { return new UTF8Encoding(); }
            set { throw new NotImplementedException(); }
        }

        /// <summary>
        /// Reads the current contents that are present in the stream and returns an object that
        /// represents them.
        /// </summary>
        /// <param name="stream">The stream from which the data is read.</param>
        /// <returns>An object that represents the data in the stream.</returns>
        public object Read(Stream stream)
        {
            int len = 0;

            len = stream.ReadByte() * 256;
            len += stream.ReadByte();
            byte[] inBuffer = new byte[len];
            stream.Read(inBuffer, 0, len);
            var json = Encoding.GetString(inBuffer);
  
            // we just need to read the string from the python code and parse it as a json string :D 
            var correctJson = (json.StartsWith("{")) ? json : json.Substring(2);
            var message = JObject.Parse(correctJson);
            return message;
        }

        /// <summary>
        /// Writes the given object to the stream so that the python code can read it.
        /// </summary>
        /// <param name="outData">The object to be serialized in the stream.</param>
        /// <param name="stream">The stream to which the serialization will be written.</param>
        /// <returns>The position in the stream after the write operation.</returns>
        public int Write(object outData, Stream stream)
        {
            // generate the json from the outData and write it ot the stream
            var json = JsonConvert.SerializeObject(outData, Formatting.None);
            byte[] outBuffer = Encoding.GetBytes(json);
            int len = outBuffer.Length;
            if (len > UInt16.MaxValue)
            {
                len = (int)UInt16.MaxValue;
            }
            stream.WriteByte((byte)(len / 256));
            stream.WriteByte((byte)(len & 255));
            stream.Write(outBuffer, 0, len);
            stream.Flush();

            return outBuffer.Length + 2;
        }

        #endregion
    }
}
