﻿/*
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.ServiceModel;
using System.Threading;
using Canonical.UbuntuOne.Common;
using Canonical.UbuntuOne.Common.Aop;
using Canonical.UbuntuOne.Common.Security;
using Canonical.UbuntuOne.Common.Utils;
using Canonical.UbuntuOne.Common.Validation;
using log4net;
using Timer = System.Timers.Timer;

namespace Canonical.UbuntuOne.ProcessDispatcher
{
    /// <summary>
    /// Implementation of the sync daemon interface that allows to manage the service from the windows service.
    /// </summary>
    internal class SyncDaemonService : ISyncDaemon
    {
        #region HelperStruct

        /// <summary>
        /// Provides the state of the auto sync to be performed.
        /// </summary>
        private struct AutoSyncState
        {
            #region Variables

            private string _path;
            private string _token;
            private string _tokenSecret;
            private string _consumerKey;
            private string _consumerSecret;
            private int _seconds;

            #endregion

            #region Properties

            /// <summary>
            /// Gets the path to auto sync.
            /// </summary>
            public string Path { get { return _path; } }

            /// <summary>
            /// Gets the ouath token.
            /// </summary>
            public string Token { get { return _token; } }

            /// <summary>
            /// Gets the oauth token secret.
            /// </summary>
            public string TokenSecret { get { return _tokenSecret; } }

            /// <summary>
            /// Gets the oauth consumer key.
            /// </summary>
            public string ConsumerKey { get { return _consumerKey; } }

            /// <summary>
            /// Gets the oauth consuer secret.
            /// </summary>
            public string ConsumerSecret { get { return _consumerSecret; } }

            public int Seconds { get { return _seconds; } }
            #endregion

            #region Constructors

            /// <summary>
            /// Creates a new state that will contain the passed data for the state.
            /// </summary>
            /// <param name="path">The path that represents the root folder to be used by the sync.</param>
            /// <param name="token">The ouath token.</param>
            /// <param name="tokenSecret">The oauth token secret.</param>
            /// <param name="consumerKey">The oauth consumer key.</param>
            /// <param name="consumerSecret">The oauth consumer secret.</param>
            /// <param name="seconds">The interval to use in seconds.</param>
            public AutoSyncState(string path, string token, string tokenSecret, string consumerKey, string consumerSecret, int seconds)
            {
                _path = path;
                _token = token;
                _tokenSecret = tokenSecret;
                _consumerKey = consumerKey;
                _consumerSecret = consumerSecret;
                _seconds = seconds;
            }

            #endregion

        }
        #endregion

        #region Variables

        // contains a map between user id and pid so that we know if the process is running already.
        private readonly IDictionary<string, IProcessManager> _processesMap;
        private readonly object _loggerLock = new object();
        private ILog _logger;
        private readonly object _autoManualSyncThreadLock = new object();
        private Thread _autoManualSyncThread;
        private bool _isAutoManualSync;
        private readonly object _isAutoManualSyncLock = new object();
        private readonly object _manualSyncIsRunningLock = new object();
        private bool _isManualSyncRunning = false;

        #endregion

        #region Properties

        /// <summary>
        /// Allows to get and set the logger that will be used.
        /// </summary>
        internal ILog Logger
        {
            get
            {
                if (_logger == null)
                {
                    lock (_loggerLock)
                    {
                        _logger = LogManager.GetLogger(typeof(SyncDaemonService));
                    }
                }
                return _logger;
            }
            set
            {
                _logger = value;
            }
        }

        /// <summary>
        /// Allows to get and set the logger that will be used.
        /// </summary>
        internal bool IsManualSyncRunning
        {
            get { return _isManualSyncRunning; }
            set
            {
                lock (_manualSyncIsRunningLock)
                {
                    _isManualSyncRunning = value;
                }
            }
        }

        /// <summary>
        /// Gets and sets the thread that is used to automatically run the manual sync.
        /// </summary>
        private Thread AutoManualSyncThread
        {
            get { return _autoManualSyncThread; }
            set
            {
                lock (_autoManualSyncThreadLock)
                {
                    _autoManualSyncThread = value;
                }
            }
        }

        /// <summary>
        /// Gets and sets a bool stating that we are performing the manual sync.
        /// </summary>
        private bool IsAutoManualSync
        {
            get { return _isAutoManualSync; }
            set
            {
                lock (_isAutoManualSyncLock)
                {
                    _isAutoManualSync = value;
                }
            }
        }

        /// <summary>
        /// Allows to get and set the manager that will manage the sync daemon process.
        /// </summary>
        internal IProcessManagerFactory ProcessManagerFactory { get; set; }

        /// <summary>
        /// Allows to get and set the authentication object that allows to ensure that the user making the
        /// call is allowed.
        /// </summary>
        internal IAuthentication Authentication { get; set; }

        /// <summary>
        /// Allows to get and set the application locator that can be used to find the required
        /// external applications.
        /// </summary>
        internal IApplicationLocator ApplicationLocator { get; set; }

        #endregion

        public SyncDaemonService()
        {
            _processesMap = new Dictionary<string, IProcessManager>();
        }


        #region Helpers


        private void AutoManualSync(object o)
        {
            var state = (AutoSyncState)o;
            // TODO: Doing this might be an issue later in the project. In this case the aouth info is kept in the thread and 
            // if there are changes, we will not know it. Is better to use the SSO lib to get the data and then perform the sync.

            // little trick, we use the using statement to ensure that the timer is kept alive while 
            // we needed but that it will be disposed when we stop the operation
            using (var manualSyncTimer = new Timer(10000))
            {
                // use a lambda as the callback to use
                manualSyncTimer.Elapsed +=
                    (sender, args) => ManualSync(state.Path, state.Token, state.TokenSecret, state.ConsumerKey, state.ConsumerSecret);

                // we got the number of seconds, therefor multiple by 1000 the value
                manualSyncTimer.Interval = state.Seconds * 1000;
                manualSyncTimer.Enabled = true;
                //stay here forever until we are killed
                while (IsAutoManualSync)
                {
                    //wait
                }
                //disable the timer
                manualSyncTimer.Enabled = false;
            }
        }

        #endregion

        /// <summary>
        /// Allows to start the daemon by a client.
        /// </summary>
        /// <exception cref="SyncDaemonException">Exception thrown when the process could not be started.</exception>
        [DebugLogged]
        [OperationBehavior(Impersonation = ImpersonationOption.Allowed)]
        public void Start()
        {
            ValidateArgs.Begin()
                .IsNotNull(Authentication)
                .Check();

            if (Authentication.IsAuthenticated)
            {
                var user = Authentication.Username;
                if (!_processesMap.ContainsKey(user))
                {
                    Logger.DebugFormat("Starting daemon for {0}.", user);
                    var manager = ProcessManagerFactory.GetManager();
                    try
                    {
                        // start the process and relate it to the user.
                        manager.Start();
                        _processesMap.Add(user, manager);
                    }
                    catch (ProcessManagementException ex)
                    {
                        var message = String.Format("Daemon for {0} could not be started: {1}", user, ex.Message);
                        Logger.ErrorFormat(message);
                        throw new SyncDaemonException(message, ex);
                    }
                }
                else
                {
                    Logger.WarnFormat("User {0} sync daemon is already running.", user);
                }
            }
        }

        /// <summary>
        /// Allows to try and start a daemon from the client. If the daemon cannot be started a false will be returned.
        /// </summary>
        /// <returns>A boolean flag telling if the daemon could be started.</returns>
        [DebugLogged]
        public bool TryStart()
        {
            // depend on the start method
            try
            {
                Start();
                return true;
            }
            catch (SyncDaemonException)
            {
                return false;
            }
        }

        /// <summary>
        /// Allows to stop the daemon by a client.
        /// </summary>
        /// <exception cref="SyncDaemonException">Exception thrown when the process could not be stopped.</exception>
        [DebugLogged]
        public void Stop()
        {
            ValidateArgs.Begin()
                .IsNotNull(Authentication)
                .Check();

            if (Authentication.IsAuthenticated)
            {
                var user = Authentication.Username;
                if (_processesMap.ContainsKey(user))
                {
                    // get the manager, stop the process and remove the info
                    var manager = _processesMap[user];
                    try
                    {
                        manager.Stop();
                        _processesMap.Remove(user);
                    }
                    catch (ProcessManagementException ex)
                    {
                        var message = String.Format("Daemon for {0} could not be stoped: {1}", user, ex.Message);
                        throw new SyncDaemonException(message, ex);
                    }
                }
                else
                {
                    Logger.WarnFormat("User {0} sync daemon is not running.", user);
                }
            }
        }

        /// <summary>
        /// Allows to try and stop the daemon from the client. If the daemon cannot be stopped a false will be returned.
        /// </summary>
        /// <returns>A boolean flag telling if the daemong could bs stopped.</returns>
        [DebugLogged]
        public bool TryStop()
        {
            try
            {
                Stop();
                return true;
            }
            catch (SyncDaemonException)
            {
                return false;
            }
        }

        /// <summary>
        /// Allows to perform a manual sync of the root Ubuntu One directory in the provided path using the oauth information
        /// passed to the service.
        /// </summary>
        /// <param name="path">The path to which the root of the Ubuntu One folder will be sync.</param>
        /// <param name="token">The token from oauth.</param>
        /// <param name="tokenSecret">The token secret from oauth.</param>
        /// <param name="consumerKey">The consumer key from oauth.</param>
        /// <param name="consumerSecret">The consumer secret from oauth.</param>
        public void ManualSync(string path, string token, string tokenSecret, string consumerKey, string consumerSecret)
        {
            ValidateArgs.Begin()
                .IsNotNullOrEmpty(path, "path")
                .IsNotNullOrEmpty(token, "toke")
                .IsNotNullOrEmpty(consumerKey, "consumerKey")
                .IsNotNullOrEmpty(consumerSecret, "consumerSecret")
                .Check();

            if (!IsManualSyncRunning)
            {
                IsManualSyncRunning = true;
                // should always be executed befre the impersonation
                var u1SyncPath = ApplicationLocator.GetApplicationPath(Constants.U1SyncAppName);
                var tokens = string.Format("{0}:{1}:{2}:{3}", token, tokenSecret, consumerKey, consumerSecret);
                if (!Directory.Exists(path))
                {
                    Directory.CreateDirectory(path);
                }
                if (!Directory.Exists(Path.Combine(path, ".ubuntuone-sync")))
                {
                    var initCommand = String.Format("{0} --init \"{1}\" --oauth {2}",
                                                    Path.Combine(u1SyncPath, "main.exe"),
                                                    path, tokens);
                    Logger.DebugFormat("Executing {0} as user.", initCommand);

                    //init the directory
                    var init = new Process
                                   {
                                       StartInfo =
                                           {
                                               WorkingDirectory = u1SyncPath,
                                               FileName = Path.Combine(u1SyncPath, "main.exe"),
                                               Arguments = String.Format(" --init \"{0}\" --oauth {1}", path, tokens),
                                               UseShellExecute = false,
                                               RedirectStandardOutput = true,
                                               RedirectStandardError = true,
                                               CreateNoWindow = true

                                           }
                                   };
                    Logger.InfoFormat("Executing u1sync with args: {0}", init.StartInfo.Arguments);
                    init.Start();
                    init.WaitForExit();
                    Logger.InfoFormat("StandarOutput from u1sync is: {0}", init.StandardOutput.ReadToEnd());
                    Logger.ErrorFormat("StandardError from u1sync is: {0}", init.StandardError.ReadToEnd());
                    init.Close();

                }
                var sync = new Process
                               {
                                   StartInfo =
                                       {
                                           WorkingDirectory = u1SyncPath,
                                           FileName = Path.Combine(u1SyncPath, "main.exe"),
                                           Arguments = String.Format("\"{0}\" --oauth {1}", path, tokens),
                                           UseShellExecute = false,
                                           RedirectStandardOutput = true,
                                           RedirectStandardError = true,
                                           CreateNoWindow = true
                                       }
                               };
                sync.Start();
                // we wait, we are in a diff thread in a diff process this will not block the user UI
                sync.WaitForExit();
                var std_out = sync.StandardOutput.ReadToEnd();
                var std_err = sync.StandardError.ReadToEnd();
                Logger.InfoFormat("StandarOutput from u1sync is: {0}", sync.StandardOutput.ReadToEnd());
                Logger.ErrorFormat("StandardError from u1sync is: {0}", sync.StandardError.ReadToEnd());
                sync.Close();
                IsManualSyncRunning = false;
            }
        }

        /// <summary>
        /// Allows to set the manual sync operation to be performed every number of seconds.
        /// </summary>
        /// <param name="path">The path to which the root of the Ubuntu One folder will be sync.</param>
        /// <param name="token">The token from oauth.</param>
        /// <param name="tokenSecret">The token secret from oauth.</param>
        /// <param name="consumerKey">The consumer key from oauth.</param>
        /// <param name="consumerSecret">The consumer secret from oauth.</param>
        /// <param name="numberOfSeconds">The number of seconds that the timer will wiat everytime it executes the sync.</param>
        public void EnableManualSyncAutoExecution(string path, string token, string tokenSecret, string consumerKey, string consumerSecret, int numberOfSeconds)
        {
            if (AutoManualSyncThread != null)
                // we are already doing it, ignore
                return;
            Logger.InfoFormat("Enabling auto sync every {0} seconds", numberOfSeconds);
            var state = new AutoSyncState(path, token, tokenSecret, consumerKey, consumerSecret, numberOfSeconds);
            // we set var to tell that we are going to perform the auto execution
            IsAutoManualSync = true;
            // we create the tread
            AutoManualSyncThread = new Thread(AutoManualSync) { Name = "AutoManualSyncThread" };
            AutoManualSyncThread.Start(state);
        }

        /// <summary>
        /// Allows to disable the auto execution of the manual sync execution. 
        /// </summary>
        public void DisableManualSyncAutoExecution()
        {
            Logger.InfoFormat("Disabling autosync.");
            IsAutoManualSync = false;
            AutoManualSyncThread = null;
        }

        /// <summary>
        /// Returns the port number of the socket that the synd daemo is listening to for communication.
        /// </summary>
        /// <returns></returns>
        public int GetListeningSocket()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Allows to query the service about the current status of the daemon.
        /// </summary>
        /// <returns>A data contract that contains the current status of the daemon.</returns>
        [DebugLogged]
        public StateData GetCurrentStatus()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Allows to query the service to return the downloads that are currently being performed.
        /// </summary>
        /// <returns>A list with all the current downloads being processed by the daemon.</returns>
        [DebugLogged]
        public IList<DownloadData> GetCurrentDownloads()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Allows to query the service to return the items that are either waiting from upload or download.
        /// </summary>
        /// <returns>The list of items that are in the queue to be processed.</returns>
        [DebugLogged]
        public IList<QueueItemData> GetWaitingContent()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Allows the client to schedule the share and node be next in the
        /// queue of waiting commands.
        /// </summary>
        /// <param name="shareId">The unique id that describes the share.</param>
        /// <param name="nodeId">The unique id that describes the node.</param>
        [DebugLogged]
        public void ScheduleAsNext(string shareId, string nodeId)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Allows to query the aemon to retrieve all the different nodes that are currently neing uploaded.
        /// </summary>
        /// <returns>The list of the objects currently being uploaded.</returns>
        [DebugLogged]
        public IList<UploadData> GetCurrentUploads()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Allows the client to tell the daemon to connect. The daemon should not connect automatically, this
        /// should be explicitly stated by the client.
        /// </summary>
        [DebugLogged]
        public void Connect()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Allows the client to tell the daemon to disconnect.
        /// </summary>
        [DebugLogged]
        public void Disconnect()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Allows the client to query the root directoy used by the daemon.
        /// </summary>
        /// <returns></returns>
        [DebugLogged]
        public string GetRootDirectory()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Allows client to tell the sync daemon to quit. Discomment allows the daemon to quee the event
        /// for the next time it connects, quit completelly closes the service and stops listening the IO changes.
        /// </summary>
        [DebugLogged]
        public void Quit()
        {
            throw new NotImplementedException();
        }
    }
}
