﻿/*
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System.ServiceModel;
using Canonical.UbuntuOne.Common.Aop;
using log4net;

namespace Canonical.UbuntuOne.ProcessDispatcher
{
    /// <summary>
    /// This class represents the windows service that is able to launch the sync daemon accoring to the user.
    /// 
    /// This service is used to ensure that the sync daemon run sper user and allows to amange those processes.
    /// </summary>
    public class SyncDaemonWindowsService
    {
        #region Variables

        ///<summary>
        /// The name that uniquely identifies the service.
        ///</summary>
        public const string MyServiceName = "UbuntuOne SyncDaemon";
        private readonly object _loggerLock = new object();
        private ILog _logger;

        #endregion

        #region Properties

        /// <summary>
        /// Gets and sets the logger that will be used to log the operation of the
        /// class.
        /// </summary>
        public ILog Logger
        {
            get
            {
                if (_logger == null)
                {
                    lock (_loggerLock)
                    {
                        _logger = LogManager.GetLogger(typeof(SyncDaemonWindowsService));
                    }
                }
                return _logger;
            }
            set
            {
                _logger = value;
            }
        }

        #endregion

        #region DI Properties

        /// <summary>
        /// Gets and sets the service host that will be used to manage the 
        /// configuration of the service.
        /// </summary>
        public ServiceHost SyncConfiguration { get; set; }

        /// <summary>
        /// Gets and sets the service host that will be used to interact 
        /// with the sync daemon.
        /// </summary>
        public ServiceHost SyncDaemon { get; set; }

        /// <summary>
        /// Gets and sets the service host that can be used to interact with 
        /// the file manager of the sync daemon.
        /// </summary>
        public ServiceHost SyncFileManager { get; set; }

        /// <summary>
        /// Gets and sets the service host that can be used to interact with 
        /// the folders from the sync daemon.
        /// </summary>
        public ServiceHost SyncFolders { get; set; }

        /// <summary>
        /// Gets and sets the service hots that can be used to interact with the 
        /// shares in the sync daemon.
        /// </summary>
        public ServiceHost SyncShares { get; set; }

        #endregion

        #region Helpers

        private ServiceHost[] GetHosts()
        {
            return new[]
            {
                SyncConfiguration,
                SyncDaemon,
                SyncFileManager,
                SyncFolders,
                SyncShares
            };
        }

        #endregion

        /// <summary>
        /// Start this service.
        /// </summary>
        [DebugLogged]
        public void Start()
        {

            foreach (var serviceHost in GetHosts())
            {
                // make sure that it is close and reopen
                Logger.InfoFormat("Host {0} state is {1}", serviceHost.BaseAddresses[0], serviceHost.State);
                if (serviceHost.State == CommunicationState.Closed)
                    serviceHost.Open();
            }
            Logger.DebugFormat("{0} started.", MyServiceName);
        }

        /// <summary>
        /// Stop this service.
        /// </summary>
        [DebugLogged]
        public void Stop()
        {
            var hosts = GetHosts();

            for (var index = 0; index < hosts.Length; index++)
            {
                if (hosts[index] == null)
                {
                    continue;
                }
                hosts[index].Close();
                hosts[index] = null;
            }
            Logger.DebugFormat("{0} stopped", MyServiceName);
        }
    }
}
