﻿/*
 * Copyright 2010 Canonical Ltd.
 * 
 * This file is part of Ubuntu One on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Peña <manuel.delapena@canonical.com>
 */
using System;
using System.Security.Principal;
using System.Threading;
using System.Windows;
using Canonical.UbuntuOne.Client.Notification;
using Canonical.UbuntuOne.Client.Preferences;
using Canonical.UbuntuOne.Common.Container;
using Canonical.UbuntuOne.Common.Update;
using Canonical.UbuntuOne.Common.Utils;
using Canonical.UbuntuOne.ProcessDispatcher;
using log4net;

namespace UbuntuOneClient
{
    /// <summary>
    /// Class that will start the application when needed.
    /// </summary>
    internal class Program
    {
        private static readonly ILog _logger = LogManager.GetLogger(typeof(Program));
        private IUpdater _updater;
        private readonly object _updaterLock = new object();
        private INotificationIconView _notifyIcon;
        private readonly object _notifyIconLock = new object();
        private Thread _updatesThread;

        #region DI Properties for testability

        /// <summary>
        /// Allows to get and set the updater to be used, if the updater is not present the class
        /// will use the ObjectsContainer to retrieve it. This property has been added to increase the testability
        /// of the class.
        /// </summary>
        internal IUpdater Updater
        {
            get
            {
                if (_updater == null)
                {
                    lock (_updaterLock)
                    {
                        _updater = ObjectsContainer.GetImplementationOf<IUpdater>();
                    }
                }
                return _updater;
            }
            set { _updater = value; }
        }

        /// <summary>
        /// Allows to get and set the notification icon, if the notification icon is not present the class 
        /// will use the Objects container to retrieve it. This property has been added to icrease the testability of the 
        /// class.
        /// </summary>
        internal INotificationIconView NotifyIcon
        {
            get
            {
                if (_notifyIcon == null)
                {
                    lock (_notifyIconLock)
                    {
                        _notifyIcon = ObjectsContainer.GetImplementationOf<INotificationIconView>();
                    }
                }
                return _notifyIcon;
            }
            set { _notifyIcon = value; }
        }

        /// <summary>
        /// Allows to get and set the application that will be used to ran the client. This property has been 
        /// added to increase the testability of the class.
        /// </summary>
        internal IApplication Application { get; set; }

        #endregion

        #region Helper methods

        /// <summary>
        /// Helper method that will check for updates and will ask the user if an update is required.
        /// </summary>
        private void CheckForUpdates()
        {
            try
            {
                if (!Updater.UpdatesArePresent()) return;
                _logger.Info("Updates are available.");
                Updater.PerformUpdate();
            }
            catch (SelfUpdateException ex)
            {
                _logger.WarnFormat("Application could not be update {0}", ex);
            }
        }

        #endregion

        /// <summary>
        /// Starts the application when requested. The code has been refactored to this method to allow
        /// unit testing.
        /// </summary>
        public void StartApplication()
        {
            _logger.Debug("Starting client application.");
            // we explicitly want to stop the application
            Application.ShutdownMode = ShutdownMode.OnExplicitShutdown;
            // we start the check for updates in another thread so that we do not block the user, 
            // that way we will not stop the execution of the application while we chec the rss feed.
            _updatesThread = new Thread(CheckForUpdates);
            _updatesThread.Start();
            
            // we do not worry about looping to allow the other thread to finish, we no that app should take longer
            Application.Run(NotifyIcon as Window);
            
        }

        /// <summary>
        /// Allows to stop the trhad that is checking if updates are avialble.
        /// </summary>
        public void StopUpdaterThread()
        {
            if (_updatesThread != null)
            {
                _updatesThread.Join();
            }
            else
            {
                _logger.Warn("Not update thread exists.");
            }
        }


        [STAThread]
        static void Main()
        {
            _logger.Info("Updating config");
            // init the objects container
            // Setup information for the new appdomain.

            ObjectsContainer.Initialize(new SpringContainer());
            var program = ObjectsContainer.GetImplementationOf<Program>();
            // HACK: This should be done by the IoC and not manually
            program.Application.StartupTasks.Add(ObjectsContainer.GetImplementationOf<PipeListeningStartupTask>());
            program.Application.StartupTasks.Add(ObjectsContainer.GetImplementationOf<LoadPreferencesStartupTask>());
            program.StartApplication();

        }
    }

}
