# ubuntuone.u1sync.main
#
# Prototype directory sync client
#
# Author: Lucio Torre <lucio.torre@canonical.com>
# Author: Tim Cole <tim.cole@canonical.com>
#
# Copyright 2009 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""A prototype directory sync client."""

from __future__ import with_statement

import os
import sys
import uuid
import signal
import logging
from Queue import Queue
from errno import EEXIST
from twisted.internet import reactor
# import the storeage protocol to allow the communication with the server
import ubuntuone.storageprotocol.dircontent_pb2 as dircontent_pb2
from ubuntuone.storageprotocol.dircontent_pb2 import DIRECTORY, SYMLINK
# import helper modules
from u1sync.genericmerge import (
    show_tree, generic_merge)
from u1sync.client import (
    ConnectionError, AuthenticationError, NoSuchShareError,
    ForcedShutdown, Client)
from u1sync.scan import scan_directory
from u1sync.merge import (
    SyncMerge, ClobberServerMerge, ClobberLocalMerge, merge_trees)
from u1sync.sync import download_tree, upload_tree
from u1sync.utils import safe_mkdir, send_bus_error, send_notification
from u1sync import metadata
from u1sync.constants import METADATA_DIR_NAME
from u1sync.ubuntuone_optparse import UbuntuOneOptionsParser

# pylint: disable-msg=W0212
NODE_TYPE_ENUM = dircontent_pb2._NODETYPE
# pylint: enable-msg=W0212
def node_type_str(node_type):
    """Converts a numeric node type to a human-readable string."""
    return NODE_TYPE_ENUM.values_by_number[node_type].name


class ReadOnlyShareError(Exception):
    """Share is read-only."""


class DirectoryAlreadyInitializedError(Exception):
    """The directory has already been initialized."""


class DirectoryNotInitializedError(Exception):
    """The directory has not been initialized."""


class NoParentError(Exception):
    """A node has no parent."""


class TreesDiffer(Exception):
    """Raised when diff tree differs."""
    def __init__(self, quiet):
        self.quiet = quiet


MERGE_ACTIONS = {
    # action: (merge_class, should_upload, should_download)
    'sync': (SyncMerge, True, True),
    'clobber-server': (ClobberServerMerge, True, False),
    'clobber-local': (ClobberLocalMerge, False, True),
    'upload': (SyncMerge, True, False),
    'download': (SyncMerge, False, True),
    'auto': None # special case
}

DEFAULT_MERGE_ACTION = 'auto'

def do_init(client, share_spec, directory, quiet, subtree_path,
            metadata=metadata):
    """Initializes a directory for syncing, and syncs it."""
    info = metadata.Metadata()

    if share_spec is not None:
        info.share_uuid = client.find_volume(share_spec)
    else:
        info.share_uuid = None

    if subtree_path is not None:
        info.path = subtree_path
    else:
        info.path = "/"

    logging.info("Initializing directory...")
    safe_mkdir(directory)

    metadata_dir = os.path.join(directory, METADATA_DIR_NAME)
    try:
        os.mkdir(metadata_dir)
    except OSError, e:
        if e.errno == EEXIST:
            raise DirectoryAlreadyInitializedError(directory)
        else:
            raise

    logging.info("Writing mirror metadata...")
    metadata.write(metadata_dir, info)

    logging.info("Done.")

def do_sync(client, directory, action, dry_run, quiet):
    """Synchronizes a directory with the given share."""
    absolute_path = os.path.abspath(directory)
    while True:
        metadata_dir = os.path.join(absolute_path, METADATA_DIR_NAME)
        if os.path.exists(metadata_dir):
            break
        if absolute_path == "/":
            raise DirectoryNotInitializedError(directory)
        absolute_path = os.path.split(absolute_path)[0]

    send_notification("Reading", "Reading mirror metadata.", "info")
    logging.info("Reading mirror metadata...")
    info = metadata.read(metadata_dir)

    top_uuid, writable = client.get_root_info(info.share_uuid)

    if info.root_uuid is None:
        info.root_uuid = client.resolve_path(info.share_uuid, top_uuid,
                                             info.path)

    if action == 'auto':
        if writable:
            action = 'sync'
        else:
            action = 'download'
    merge_type, should_upload, should_download = MERGE_ACTIONS[action]
    if should_upload and not writable:
        raise ReadOnlyShareError(info.share_uuid)
        
    logging.info("Scanning directory...")
    send_notification("Scanning", "Scanning directory...", "info")
    
    local_tree = scan_directory(absolute_path, quiet=quiet)
 
    logging.info("Fetching metadata...")
    send_notification("Scanning", "Fetching metadata...", "info")
    
    remote_tree = client.build_tree(info.share_uuid, info.root_uuid)
    if not quiet:
        show_tree(remote_tree)

    logging.info("Merging trees...")
    merged_tree = merge_trees(old_local_tree=info.local_tree,
                              local_tree=local_tree,
                              old_remote_tree=info.remote_tree,
                              remote_tree=remote_tree,
                              merge_action=merge_type())
    if not quiet:
        show_tree(merged_tree)

    logging.info("Syncing content...")
    send_notification("Syncing", "Syncing content...", "info")
    if should_download:
        info.local_tree = download_tree(merged_tree=merged_tree,
                                        local_tree=local_tree,
                                        client=client,
                                        share_uuid=info.share_uuid,
                                        path=absolute_path, dry_run=dry_run,
                                        quiet=quiet)
    else:
        info.local_tree = local_tree
    if should_upload:
        info.remote_tree = upload_tree(merged_tree=merged_tree,
                                       remote_tree=remote_tree,
                                       client=client,
                                       share_uuid=info.share_uuid,
                                       path=absolute_path, dry_run=dry_run,
                                       quiet=quiet)
    else:
        info.remote_tree = remote_tree

    if not dry_run:
        logging.info("Updating mirror metadata...")
        send_notification("Syncing", "Updating mirror metadata...", "info")
        metadata.write(metadata_dir, info)

    logging.info("Done.")

def do_list_shares(client):
    """Lists available (incoming) shares."""
    shares = client.get_incoming_shares()
    for (name, id, user, accepted, access) in shares:
        if not accepted:
            status = " [not accepted]"
        else:
            status = ""
        print "%s  %s (from %s) [%s]%s" % (id, name, user, access, status)

def do_diff(client, share_spec, directory, quiet, subtree_path,
            ignore_symlinks=True):
    """Diffs a local directory with the server."""
    if share_spec is not None:
        share_uuid = client.find_volume(share_spec)
    else:
        share_uuid = None
    if subtree_path is None:
        subtree_path = '/'
    # pylint: disable-msg=W0612
    root_uuid, writable = client.get_root_info(share_uuid)
    subtree_uuid = client.resolve_path(share_uuid, root_uuid, subtree_path)
    local_tree = scan_directory(directory, quiet=True)
    remote_tree = client.build_tree(share_uuid, subtree_uuid)

    def pre_merge(nodes, name, partial_parent):
        """Compares nodes and prints differences."""
        (local_node, remote_node) = nodes
        # pylint: disable-msg=W0612
        (parent_display_path, parent_differs) = partial_parent
        display_path = os.path.join(parent_display_path, name)
        differs = True
        if local_node is None:
            logging.info("%s missing from client",display_path)
        elif remote_node is None:
            if ignore_symlinks and local_node.node_type == SYMLINK:
                differs = False
                logging.info("%s missing from server",display_path)
        elif local_node.node_type != remote_node.node_type:
            local_type = node_type_str(local_node.node_type)
            remote_type = node_type_str(remote_node.node_type)
            logging.info("%s node types differ (client: %s, server: %s)",
                display_path, local_type, remote_type)
        elif local_node.node_type != DIRECTORY and \
             local_node.content_hash != remote_node.content_hash:
            local_content = local_node.content_hash
            remote_content = remote_node.content_hash
            logging.info("%s has different content (client: %s, server: %s)",
                display_path, local_content, remote_content)
        else:
            differs = False
        return (display_path, differs)

    def post_merge(nodes, partial_result, child_results):
        """Aggregates 'differs' flags."""
        # pylint: disable-msg=W0612
        (display_path, differs) = partial_result
        return differs or any(child_results.itervalues())

    differs = generic_merge(trees=[local_tree, remote_tree],
                            pre_merge=pre_merge, post_merge=post_merge,
                            partial_parent=("", False), name=u"")
    if differs:
        raise TreesDiffer(quiet=quiet)

def do_main(argv, options_parser):
    """The main user-facing portion of the script."""
    # pass the arguments to ensure that the code works correctly
    options_parser.get_options(argv)
    client = Client(realm=options_parser.options.realm, reactor=reactor)
    # set the logging level to info in the user passed quite to be false
    if options_parser.options.quiet:
        logging.basicConfig(level=logging.INFO)
        
    signal.signal(signal.SIGINT, lambda s, f: client.force_shutdown())
    signal.signal(signal.SIGTERM, lambda s, f: client.force_shutdown())

    def run_client():
        """Run the blocking client."""
        token = options_parser.options.token
        client.connect_ssl(options_parser.options.host, 
            int(options_parser.options.port), 
            options_parser.options.no_ssl_verify)

        try:
            client.set_capabilities()
            client.oauth_from_token(options_parser.options.token, options_parser.options.consumer)

            if options_parser.options.mode == "sync":
                do_sync(client=client, directory=options_parser.options.directory,
                        action=options_parser.options.action,
                        dry_run=options_parser.options.dry_run, 
                        quiet=options_parser.options.quiet)
            elif options_parser.options.mode == "init":
                do_init(client=client, share_spec=options_parser.options.share,
                        directory=options_parser.options.directory,
                        quiet=options_parser.options.quiet, subtree_path=options_parser.options.subtree)
            elif options.mode == "list-shares":
                do_list_shares(client=client)
            elif options.mode == "diff":
                do_diff(client=client, share_spec=options_parser.options.share,
                        directory=directory,
                        quiet=options_parser.options.quiet, 
                        subtree_path=options_parser.options.subtree,
                        ignore_symlinks=False)
            elif options_parser.options.mode == "authorize":
                if not options_parser.options.quiet:
                    print "Authorized."
        finally:
            client.disconnect()

    def capture_exception(queue, func):
        """Capture the exception from calling func."""
        try:
            func()
        except Exception:
            queue.put(sys.exc_info())
        else:
            queue.put(None)
        finally:
            reactor.callWhenRunning(reactor.stop)

    queue = Queue()
    reactor.callInThread(capture_exception, queue, run_client)
    reactor.run(installSignalHandlers=False)
    exc_info = queue.get(True, 0.1)
    if exc_info:
        raise exc_info[0], exc_info[1], exc_info[2]

def main(argv):
    """Top-level main function."""
    try:
        do_main(argv, UbuntuOneOptionsParser())
    except AuthenticationError, e:
        print "Authentication failed: %s" % e
    except ConnectionError, e:
        print "Connection failed: %s" % e
    except DirectoryNotInitializedError, e:
        print "Directory not initialized; " \
            "use --init [DIRECTORY] to initialize it."
    except DirectoryAlreadyInitializedError, e:
        print "Directory already initialized."
    except NoSuchShareError, e:
        print "No matching share found."
    except ReadOnlyShareError, e:
        print "The selected action isn't possible on a read-only share."
    except (ForcedShutdown, KeyboardInterrupt), e:
        print "Interrupted!"
    except TreesDiffer, e:
        if not e.quiet:
            print "Trees differ."
    else:
        return 0
    return 1

if __name__ == "__main__":
    # set the name of the process, this just works on windows
    sys.argv[0] = "Ubuntuone"
    sys.exit(main(sys.argv[1:]))