# ubuntuone.u1sync.ubuntuone_optparse
#
# Prototype directory sync client
#
# Author: Manuel de la Pena <manuel.delapena@canonical.com>
#
# Copyright 2010 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
import uuid
from oauth.oauth import OAuthToken, OAuthConsumer
from optparse import OptionParser, SUPPRESS_HELP
from u1sync.merge import (
    SyncMerge, ClobberServerMerge, ClobberLocalMerge)

MERGE_ACTIONS = {
    # action: (merge_class, should_upload, should_download)
    'sync': (SyncMerge, True, True),
    'clobber-server': (ClobberServerMerge, True, False),
    'clobber-local': (ClobberLocalMerge, False, True),
    'upload': (SyncMerge, True, False),
    'download': (SyncMerge, False, True),
    'auto': None # special case
}

DEFAULT_MERGE_ACTION = 'auto'

class NotParsedOptionsError(Exception):
    """Exception thrown when there options have not been parsed."""
    
class NotValidatedOptionsError(Exception):
    """Exception thrown when the options have not been validated."""
    
class UbuntuOneOptionsParser(OptionParser):
    """Parse for the options passed in the command line"""
    
    def __init__(self):
        usage = "Usage: %prog [options] [DIRECTORY]\n" \
                "       %prog --authorize [options]\n" \
                "       %prog --list-shares [options]\n" \
                "       %prog --init [--share=SHARE_UUID] [options] DIRECTORY\n" \
                "       %prog --diff [--share=SHARE_UUID] [options] DIRECTORY"
        OptionParser.__init__(self, usage=usage)
        self._was_validated = False
        self._args = None
        # add the different options to be used
        self.add_option("--port", dest="port", metavar="PORT",
                          default=443,
                          help="The port on which to connect to the server")
        self.add_option("--host", dest="host", metavar="HOST",
                          default='fs-1.one.ubuntu.com',
                          help="The server address")
        self.add_option("--realm", dest="realm", metavar="REALM",
                          default='https://ubuntuone.com',
                          help="The oauth realm")
        self.add_option("--oauth", dest="oauth", metavar="KEY:SECRET",
                          default=None,
                          help="Explicitly provide OAuth credentials "
                               "(default is to query keyring)")
                               
        action_list = ", ".join(sorted(MERGE_ACTIONS.keys()))
        self.add_option("--action", dest="action", metavar="ACTION",
                          default=None,
                          help="Select a sync action (%s; default is %s)" % \
                               (action_list, DEFAULT_MERGE_ACTION))
                               
        self.add_option("--dry-run", action="store_true", dest="dry_run",
                          default=False, help="Do a dry run without actually "
                                              "making changes")
        self.add_option("--quiet", action="store_true", dest="quiet",
                          default=False, help="Produces less output")
        self.add_option("--authorize", action="store_const", dest="mode",
                          const="authorize",
                          help="Authorize this machine")
        self.add_option("--list-shares", action="store_const", dest="mode",
                          const="list-shares", default="sync",
                          help="List available shares")
        self.add_option("--init", action="store_const", dest="mode",
                          const="init",
                          help="Initialize a local directory for syncing")
        self.add_option("--no-ssl-verify", action="store_true",
                          dest="no_ssl_verify",
                          default=False, help=SUPPRESS_HELP)
        self.add_option("--diff", action="store_const", dest="mode",
                          const="diff",
                          help="Compare tree on server with local tree " \
                               "(does not require previous --init)")
        self.add_option("--share", dest="share", metavar="SHARE_UUID",
                          default=None,
                          help="Sync the directory with a share rather than the " \
                               "user's own volume")
        self.add_option("--subtree", dest="subtree", metavar="PATH",
                          default=None,
                          help="Mirror a subset of the share or volume")
    
    def get_options(self, arguments):
        """Parses the arguments to from the command line."""
        (self.options, self._args) = \
            self.parse_args(arguments)
        self._validate_args()
    
    def _validate_args(self):
        """Validates the args that have been parsed."""
        self._is_only_share()
        self._get_directory()
        self._validate_action_usage()
        self._validate_authorize_usage()
        self._validate_subtree_usage()
        self._validate_action()
        self._validate_oauth()
        
    def _is_only_share(self):
        """Ensures that the share options is not convined with any other."""
        if self.options.share is not None and \
                self.options.mode != "init" and \
                self.options.mode != "diff":
            self.error("--share is only valid with --init or --diff")
    
    def _get_directory(self):
        """Gets the directory to be used according to the paramenters."""
        print self._args
        if self.options.mode == "sync" or self.options.mode == "init" or \
           self.options.mode == "diff":
            if len(self._args) > 2:
                self.error("Too many arguments")
            elif len(self._args) < 1:
                if self.options.mode == "init" or self.options.mode == "diff":
                    self.error("--%s requires a directory to "
                                 "be specified" % self.options.mode)
                else:
                    self.options.directory = "."
            else:
                self.options.directory = self._args[0]
    
    def _validate_action_usage(self):
        """Ensures that the --action option is correctly used"""
        if self.options.mode == "init" or \
            self.options.mode == "list-shares" or \
            self.options.mode == "diff" or \
            self.options.mode == "authorize":
            if self.options.action is not None:
                self.error("--%s does not take the --action parameter" % \
                             self.options.mode)
            if self.options.dry_run:
                self.error("--%s does not take the --dry-run parameter" % \
                             self.options.mode)
    
    def _validate_authorize_usage(self):
        """Validates the usage of the authorize option.""" 
        if self.options.mode == "authorize":
            if self.options.oauth is not None:
                self.error("--authorize does not take the --oauth parameter")
        if self.options.mode == "list-shares" or \
            self.options.mode == "authorize":
            if len(self._args) != 0:
                self.error("--list-shares does not take a directory")
    
    def _validate_subtree_usage(self):
        """Validates the usage of the subtree option"""
        if self.options.mode != "init" and self.options.mode != "diff":
            if self.options.subtree is not None:
                self.error("--%s does not take the --subtree parameter" % \
                    self.options.mode)
    
    def _validate_action(self):
        """Validates the actions passed to the options."""
        if self.options.action is not None and \
            self.options.action not in MERGE_ACTIONS:
            self.error("--action: Unknown action %s" % self.options.action)

        if self.options.action is None:
            self.options.action = DEFAULT_MERGE_ACTION
    
    def _validate_oauth(self):
        """Validates that the oatuh was passed."""
        if self.options.oauth is None:
            self.error("--oauth is currently compulsery.")
        else:
            try:
                (token_key, toke_secret, consumer_key, consumer_secret) = self.options.oauth.split(':', 4)
            except ValueError:
                self.error("--oauth requires a key and secret together in the "
                             " form KEY:SECRET")
            self.options.token = OAuthToken(token_key, toke_secret)
            self.options.consumer = OAuthConsumer(consumer_key, consumer_secret)
            
    def _validate_share(self):
        """Validates the share option"""
        if self.options.share is not None:
            try:
                uuid.UUID(self.options.share)
            except ValueError, e:
                self.error("Invalid --share argument: %s" % e)